@@ -78,6 +78,7 @@ static short mapmaxx;
 static short mapminy;
 static short mapmaxy;
 
+static int show_progress = 1;
 static unsigned char pcnt;
 
 static struct Seg *PickNode_traditional(struct Seg *);
@@ -125,8 +126,9 @@ static int CreateSSector(struct Seg *);
 
 static void progress()
 {
-	if(!((++pcnt)&31))
-		fprintf(stderr,"%c\b","/-\\|"[((pcnt)>>5)&3]);
+	if(show_progress)
+		if(!((++pcnt)&31))
+			fprintf(stderr,"%c\b","/-\\|"[((pcnt)>>5)&3]);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -648,19 +650,21 @@ static void sortlump(struct lumplist **link)
 
 void usage(void)
 {
- printf("\nThis Node builder was created from the basic theory stated in DEU5 (OBJECTS.C)\n"
-        "\nCredits should go to :-\n"
+ fprintf(stdout,
+        "\nThis Node builder was created from the basic theory stated in DEU5 (OBJECTS.C)\n"
+        "Credits should go to :-\n"
         "Matt Fell      (msfell@aol.com) for the Doom Specs.\n"
         "Raphael Quinet (Raphael.Quinet@eed.ericsson.se) for DEU and the original idea.\n"
         "Mark Harrison  (harrison@lclark.edu) for finding a bug in 1.1x\n"
         "Jim Flynn      (jflynn@pacbell.net) for many good ideas and encouragement.\n"
         "Jan Van der Veken for finding invisible barrier bug.\n"
+	"Colin Phipps for two bug fixes (http://lxdoom.linuxgames.com/notes.html#bsp).\n"
 #ifdef MSDOS
         "\nUsage: BSP [options] input.wad [[-o] <output.wad>]\n"
 #else
         "\nUsage: BSP [options] input.wad [-o <output.wad>]\n"
 #endif
-        "       (If no output.wad is specified, TMP.WAD is written)\n\n"
+        "       (If no output.wad is specified, \"tmp.wad\" is written)\n\n"
         "Options:\n\n"
         "  -factor <nnn>  Changes the cost assigned to SEG splits\n"
         "  -vp            Attempts to prevent visplane overflows\n"
@@ -751,8 +755,14 @@ int main(int argc,char *argv[])
  struct directory *newdirec;
 
  setbuf(stdout,NULL);
+#ifndef MSDOS  /* Unix: no whirling baton if stderr is redirected */
+ if(!isatty(2))
+   show_progress=0;
+#endif
 
  puts("* Doom BSP node builder ver 2.3 (c) 1998 Colin Reed, Lee Killough *");
+ printf("* This is the modified version that comes with Yadex %s. *\n",
+     YADEX_VERSION);
 
  parse_options(argc,argv);
 
