@@ -7,34 +7,37 @@
 /*
 This file is part of Yadex.
 
-Yadex incorporates code from DEU 5.21 that was put in the public
-domain in 1994 by Raphal Quinet and Brendon Wyber.
+Yadex incorporates code from DEU 5.21 that was put in the public domain in
+1994 by Raphal Quinet and Brendon Wyber.
 
-The rest of Yadex is Copyright  1997-1999 Andr Majorel.
+The rest of Yadex is Copyright  1997-2000 Andr Majorel.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
-You should have received a copy of the GNU Library General Public
-License along with this library; if not, write to the Free
-Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307, USA.
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place, Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
 #include "yadex.h"
-#include <errno.h>
 #include "cfgfile.h"
 #include "gfx.h"
 #include "help1.h"
 #include "levels.h"
+#include "macro.h"
+#include "trace.h"
+#include "windim.h"
+
+
+const char config_file_magic[] = "# Yadex configuration file version 3";
 
 
 /*
@@ -43,8 +46,8 @@ Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 typedef enum
    {
    // Boolean (toggle)
-   // Receptacle is of type Bool
-   // data_ptr is of type (Bool *)
+   // Receptacle is of type bool
+   // data_ptr is of type (bool *)
    OPT_BOOLEAN,
 
    // "yes", "no", "ask"
@@ -57,14 +60,20 @@ typedef enum
    // data_ptr is of type (int *)
    OPT_INTEGER,
 
+   // Window dimension, either integer (pixels) or integer%
+   // (percentage of total display dimension)
+   // Receptacle is of type ...
+   // data_ptr is of type ...
+   OPT_WINDIM,
+
    // String
    // Receptacle is of type (char[9])
    // data_ptr is of type (char *)
    OPT_STRINGBUF8,
 
    // String
-   // Receptacle is of type (char *)
-   // data_ptr is of type (char **)
+   // Receptacle is of type (const char *)
+   // data_ptr is of type (const char **)
    OPT_STRINGPTR,
 
    // String, but store in a list
@@ -73,8 +82,8 @@ typedef enum
    OPT_STRINGPTRACC,
 
    // List of strings
-   // Receptacle is of type ??
-   // data_ptr is of type ??
+   // Receptacle is of type (const char *[])
+   // data_ptr is of type (const char ***)
    OPT_STRINGPTRLIST,
 
    // End of the options description
@@ -126,6 +135,13 @@ opt_desc_t options[] =		/* description of the command line options */
   "Max. dist. to edge (pixels)",
   &autoscroll_edge },
 
+{ NULL,
+  "b",
+  OPT_STRINGPTR,
+  NULL,
+  "Run benchmark and exit successfully",
+  &bench },
+
 { "bgi",
   NULL,
   OPT_STRINGPTR,
@@ -232,7 +248,7 @@ opt_desc_t options[] =		/* description of the command line options */
   &double_click_timeout },
 
 { "expert",
-  "e",
+  NULL,
   OPT_BOOLEAN,
   NULL,
   "Expert mode",
@@ -268,7 +284,7 @@ opt_desc_t options[] =		/* description of the command line options */
 
 { "height",
   "h",
-  OPT_INTEGER,
+  OPT_WINDIM,
   "x",
   "(X11 only) Initial window height",
   &initial_window_height },
@@ -301,6 +317,13 @@ opt_desc_t options[] =		/* description of the command line options */
   "Min grid step (pixels)",
   &grid_pixels_min },
 
+{ "idle_sleep_ms",
+  NULL,
+  OPT_INTEGER,
+  NULL,
+  "ms to sleep before XPending()",
+  &idle_sleep_ms },
+
 { "info_bar",
   NULL,
   OPT_BOOLEAN,
@@ -326,14 +349,14 @@ opt_desc_t options[] =		/* description of the command line options */
   "i1",
   OPT_STRINGPTR,
   NULL,
-  "The name of Doom iwad",
+  "The name of the Doom/Ultimate D. iwad",
   &Iwad1 },
 
 { "iwad2",
   "i2",
   OPT_STRINGPTR,
   NULL,
-  "The name of the Doom II iwad",
+  "The name of the Doom II/Final D. iwad",
   &Iwad2 },
 
 { "iwad3",
@@ -343,6 +366,55 @@ opt_desc_t options[] =		/* description of the command line options */
   "The name of the Heretic iwad",
   &Iwad3 },
 
+{ "iwad4",
+  "i4",
+  OPT_STRINGPTR,
+  NULL,
+  "The name of the Hexen iwad",
+  &Iwad4 },
+
+{ "iwad5",
+  "i5",
+  OPT_STRINGPTR,
+  NULL,
+  "The name of the Strife iwad",
+  &Iwad5 },
+
+{ "iwad6",
+  "i6",
+  OPT_STRINGPTR,
+  NULL,
+  "The name of the Doom alpha 0.2 iwad",
+  &Iwad6 },
+
+{ "iwad7",
+  "i7",
+  OPT_STRINGPTR,
+  NULL,
+  "The name of the Doom alpha 0.4 iwad",
+  &Iwad7 },
+
+{ "iwad8",
+  "i8",
+  OPT_STRINGPTR,
+  NULL,
+  "The name of the Doom alpha 0.5 iwad",
+  &Iwad8 },
+
+{ "iwad9",
+  "i9",
+  OPT_STRINGPTR,
+  NULL,
+  "The name of the Doom press rel. iwad",
+  &Iwad9 },
+
+{ "iwad10",
+  "i10",
+  OPT_STRINGPTR,
+  NULL,
+  "The name of the Strife 1.0 iwad",
+  &Iwad10 },
+
 #ifdef AYM_MOUSE_HACKS
 { "mouse_horizontal_sens",
   "mh",
@@ -363,7 +435,7 @@ opt_desc_t options[] =		/* description of the command line options */
   "pw",
   OPT_STRINGPTRACC,
   NULL,
-  "Patch wad file",
+  "Pwad file to load",
   &PatchWads },
 
 { "no_pixmap",
@@ -408,6 +480,13 @@ opt_desc_t options[] =		/* description of the command line options */
   "Automatic selection of 0th object",
   &Select0 },
 
+{ "sprite_scale",
+  NULL,
+  OPT_INTEGER,
+  NULL,
+  "Relative scale of sprites",
+  &sprite_scale },
+
 { "swap_buttons",
   "sb",
   OPT_BOOLEAN,
@@ -459,17 +538,24 @@ opt_desc_t options[] =		/* description of the command line options */
 
 { "width",
   "w",
-  OPT_INTEGER,
+  OPT_WINDIM,
   "x",
   "(X11 only) Initial window width",
   &initial_window_width },
 
-{ "zoom",
+{ "zoom_default",
   "z",
   OPT_INTEGER,
   NULL,
   "Initial zoom factor",
-  &InitialScale },
+  &zoom_default },
+
+{ "zoom_step",
+  NULL,
+  OPT_INTEGER,
+  NULL,
+  "Step between zoom factors (in %)",
+  &zoom_step },
 
 { NULL,
   NULL,
@@ -481,17 +567,20 @@ opt_desc_t options[] =		/* description of the command line options */
 
 
 static const char *standard_directories[] =
-  {
-  ".",
-  "~",			/* $HOME */
-  "/",			/* $Y_DIR */
-  "/usr/local/etc",
-  "/etc",
-  0
-  };
+   {
+   "./%b",
+   "~/.yadex/%v/%b",
+   "~/.yadex/%b",
+   "%i/%b",
+   "/usr/local/etc/yadex/%v/%b",
+   "/usr/local/etc/yadex/%b",
+   "/etc/yadex/%v/%b",
+   "/etc/yadex/%b",
+   0
+   };
 
 
-static void append_item_to_list (char ***list, char *item);
+static void append_item_to_list (const char ***list, const char *item);
 int parse_one_config_file (const char *filename);
 static confirm_t confirm_e2i (const char *external);
 static const char *confirm_i2e (confirm_t internal);
@@ -506,134 +595,157 @@ static const char *confirm_i2e (confirm_t internal);
  *	If <filename> is not NULL and is not an absolute name (i.e.
  *	does not start with a "/"), the standard directories are
  *	searched for a file named <filename>. <filename> may contain
- *	slashes, E.G. if you use "-config foo/yadex.cfg", the function
- *	will search for "./foo/yadex.cfg", "~/foo/yadex.cfg",
- *	"/opt/etc/foo/yadex.cfg", and so on.
+ *	slashes, E.G. if you use "-f foo/bar", the function will search
+ *	for "./yadex/<version>/foo/bar", "~/.yadex/foo/bar",
+ *	"/usr/local/etc/yadex/foo/bar", and so on.
  *
  *	If <filename> is not NULL but is an absolute name, only that
  *	file is searched.
+ *
+ *	Return 0 on success, non-zero on failure.
  */
 int parse_config_file (const char *filename)
 {
-const char **dirname;
-int is_absolute;
-char name[257];
-int failure = 1;
+bool absolute = filename && is_absolute (filename);
 
-is_absolute = filename && *filename == '/';
-
-if (is_absolute)
-   failure = parse_one_config_file (filename);
+if (absolute)
+   {
+   printf ("Using config file \"%s\".\n", filename);
+   int failure = parse_one_config_file (filename);
+   return failure;
+   }
 else
    {
-   for (dirname = standard_directories; *dirname; dirname++)
+   char name[257];
+   const char *home = getenv ("HOME");
+   for (const char **dirname = standard_directories; *dirname; dirname++)
       {
-      if (! strcmp (*dirname, "~"))
-         if (getenv ("HOME"))
-	    al_scps (name, getenv ("HOME"), sizeof name - 1);
-         else
-            continue;
-      else if (! strcmp (*dirname, "/"))
-         if (install_dir)
-            al_scps (name, install_dir, sizeof name - 1);
-         else
-            continue;
-      else
-	 al_scps (name, *dirname, sizeof name - 1);
-      al_sapc (name, '/', sizeof name - 1);
-      al_saps (name, filename ? filename : "yadex.cfg", sizeof name - 1);
-      failure = parse_one_config_file (name);
-      if (! failure)
-         break;
+      int r = macro_expand (name, sizeof name - 1, *dirname,
+	  "%b", filename ? filename : "yadex.cfg",
+	  "%i", install_dir,
+	  "%v", yadex_version,
+	  "~",  home,
+	  (const char *) 0);
+      if (r)
+	 {
+	 trace ("cfgloc", "%s: Could not expand macro #%d", *dirname, r);
+	 continue;
+	 }
+      if (file_exists (name))
+	 {
+	 trace ("cfgloc", "%s: hit", name);
+	 printf ("Using config file \"%s\".\n", name);
+	 int failure = parse_one_config_file (name);
+	 return failure;
+	 }
+      trace ("cfgloc", "%s: miss (%s)", name, strerror (errno));
       }
+   warn ("No config file found.\n");
+   return 0;
    }
-
-if (failure && is_absolute)
-   printf ("Can't open config file \"%s\" (%s)\n.", filename, strerror (errno));
-else if (failure)
-   printf ("No config file found.\n");
-else
-   printf ("Using config file \"%s\".\n", is_absolute ? filename : name);
-return failure;
 }
 
 
 /*
  *	parse_one_config_file
  *	Try to parse one particular config file.
- *	Returns 0 on success, <>0 on failure.
+ *	Return 0 on success, <>0 on failure.
  */
+#define RETURN_FAILURE do { rc = 1; goto byebye; } while (0)
 int parse_one_config_file (const char *filename)
 {
+int   rc = 0;
 FILE *cfgfile;
 char  line[1024];
 char *value;
-char *option;
 char *p;
 const opt_desc_t *o;
 
 cfgfile = fopen (filename, "r");
 if (cfgfile == NULL)
-   return 1;
-
-if (fgets (line, sizeof line, cfgfile) == NULL
-   || strcmp (line, "# Yadex configuration file version 2\n"))
    {
-   report_error ("%s is not a valid Yadex configuration file", filename);
-   report_error ("Perhaps a version mismatch ?");
-   return 1;
+   report_error ("Can't open config file \"%s\" (%s)\n.",
+      filename, strerror (errno));
+   RETURN_FAILURE;
    }
 
-while (fgets (line, sizeof line, cfgfile) != NULL)
+/* The first line of the configuration file must
+   contain exactly config_file_magic. */
+if (fgets (line, sizeof line, cfgfile) == NULL
+   || memcmp (line, config_file_magic, sizeof config_file_magic - 1)
+   || line[sizeof config_file_magic - 1] != '\n'
+   || line[sizeof config_file_magic] != '\0')
    {
-   if (line[0] == '#' || strlen (line) < 2)
-      continue;
-   if (line[strlen (line) - 1] == '\n')
-      line[strlen (line) - 1] = '\0';
-   /* skip blanks before the option name */
-   option = line;
+   report_error ("%s is not a valid Yadex configuration file", filename);
+   report_error ("Perhaps a leftover from a previous version of Yadex ?");
+   RETURN_FAILURE;
+   }
+
+/* Execute one line on each iteration. */
+for (unsigned lnum = 2; fgets (line, sizeof line, cfgfile) != NULL; lnum++)
+   {
+   /* Skip leading whitespace */
+   char *option = line;
    while (isspace (*option))
       option++;
-   /* skip the option name */
+
+   /* Skip comments */
+   if (*option == '#')
+      continue;
+
+   /* Remove trailing newline */
+   {
+   size_t len = strlen (option);
+   if (len >= 1 && option[len - 1] == '\n')
+      option[len - 1] = '\0';
+   }
+
+   /* Skip empty lines */
+   if (! *option)
+      continue;
+
+   /* Skip the option name */
    value = option;
-   while (value[0] && value[0] != '=' && !isspace (value[0]))
+   while (*value && *value != '=' && !isspace ((unsigned char) *value))
       value++;
-   if (!value[0])
+   if (! *value)
       {
-      report_error ("invalid line in %s (ends prematurely)", filename);
-      return 1;
+      report_error ("%s(%u): line ends prematurely", filename, lnum);
+      RETURN_FAILURE;
       }
-   if (value[0] == '=')
+   if (*value == '=')
       {
       /* mark the end of the option name */
-      value[0] = '\0';
+      *value = '\0';
       }
    else
       {
       /* mark the end of the option name */
-      value[0] = '\0';
+      *value = '\0';
       value++;
       /* skip blanks after the option name */
-      while (isspace (value[0]))
+      while (isspace ((unsigned char) *value))
 	 value++;
-      if (value[0] != '=')
+      if (*value != '=')
          {
-	 report_error ("invalid line in %s (no '=')", filename);
-         return 1;
+	 report_error ("%s(%u,%d): expected an \"=\"",
+	    filename, lnum, (int) (value - line) + 1);
+	 RETURN_FAILURE;
          }
       }
    value++;
    /* skip blanks after the equal sign */
-   while (isspace (value[0]))
+   while (isspace ((unsigned char) *value))
       value++;
    for (o = options + 1; ; o++)
       {
       if (o->opt_type == OPT_END)
          {
-         report_error ("Invalid option in %s: \"%s\"", filename, option);
-         return 1;
+	 warn ("%s(%u): invalid variable \"%s\" - skipping\n",
+	   filename, lnum, option);
+	 goto next_line;
          }
-      if (!stricmp (option, o->long_name))
+      if (o->long_name && ! strcmp (option, o->long_name))
 	 {
 	 if (o->flags != NULL && strchr (o->flags, '1'))
 	    break;
@@ -648,23 +760,23 @@ while (fgets (line, sizeof line, cfgfile) != NULL)
 	 switch (o->opt_type)
 	    {
 	    case OPT_BOOLEAN:
-	       if (!stricmp (value, "yes") || !stricmp (value, "true")
-	        || !stricmp (value, "on") || !stricmp (value, "1"))
+	       if (! strcmp (value, "yes") || ! strcmp (value, "true")
+	        || ! strcmp (value, "on") || ! strcmp (value, "1"))
 		  {
 		  if (o->data_ptr)
-		     *((Bool *) (o->data_ptr)) = 1;
+		     *((bool *) (o->data_ptr)) = true;
 		  }
-	       else if (!stricmp (value, "no") || !stricmp (value, "false")
-	             || !stricmp (value, "off") || !stricmp (value, "0"))
+	       else if (! strcmp (value, "no") || ! strcmp (value, "false")
+	             || ! strcmp (value, "off") || ! strcmp (value, "0"))
 		  {
 		  if (o->data_ptr)
-		     *((Bool *) (o->data_ptr)) = 0;
+		     *((bool *) (o->data_ptr)) = false;
 		  }
 	       else
                   {
-		  report_error ("invalid value for option %s: \"%s\"",
-		    option, value);
-                  return 1;
+		  report_error ("%s(%u): invalid value for option %s: \"%s\"",
+		    filename, lnum, option, value);
+		  RETURN_FAILURE;
                   }
 	       break;
             case OPT_CONFIRM:
@@ -675,6 +787,15 @@ while (fgets (line, sizeof line, cfgfile) != NULL)
 	       if (o->data_ptr)
 	          *((int *) (o->data_ptr)) = atoi (value);
 	       break;
+	    case OPT_WINDIM:
+	       if (o->data_ptr)
+		  if (((Win_dim *) (o->data_ptr))->set (value))
+		     {
+		     report_error ("%s(%u): bad dimension spec \"%s\"",
+		        filename, lnum, value);
+		     RETURN_FAILURE;
+		     }
+	       break;
             case OPT_STRINGBUF8:
                if (o->data_ptr)
                   al_scps ((char *) o->data_ptr, value, 8);
@@ -689,13 +810,13 @@ while (fgets (line, sizeof line, cfgfile) != NULL)
 	       p = (char *) GetMemory (strlen (value) + 1);
 	       strcpy (p, value);
 	       if (o->data_ptr)
-	          append_item_to_list ((char ***) o->data_ptr, p);
+	          append_item_to_list ((const char ***) o->data_ptr, p);
 	       break;
 	    case OPT_STRINGPTRLIST:
 	       while (value[0])
 		  {
 		  option = value;
-		  while (option[0] && !isspace (option[0]))
+		  while (option[0] && !isspace ((unsigned char) option[0]))
 		     option++;
 		  option[0] = '\0';
 		  option++;
@@ -704,22 +825,27 @@ while (fgets (line, sizeof line, cfgfile) != NULL)
 		  p = (char *) GetMemory (strlen (value) + 1);
 		  strcpy (p, value);
 	          if (o->data_ptr)
-		     append_item_to_list ((char ***) o->data_ptr, p);
+		     append_item_to_list ((const char ***) o->data_ptr, p);
 		  value = option;
 		  }
 	       break;
 	    default:
                {
-               report_error ("unknown option type (BUG!)");
-               return 1;
+               report_error ("%s(%u): unknown option type %d (BUG!)",
+	         filename, lnum, (int) o->opt_type);
+	       RETURN_FAILURE;
                }
 	    }
 	 break;
 	 }
       }
+   next_line:;
    }
-fclose (cfgfile);
-return 0;
+
+byebye:
+if (cfgfile != 0)
+   fclose (cfgfile);
+return rc;
 }
 
 
@@ -751,11 +877,11 @@ while (argc > 0)
 	 {
 	 if (o->opt_type == OPT_END)
 	    {
-	    report_error ("invalid argument: \"%s\"", argv[0]);
+	    report_error ("invalid option: \"%s\"", argv[0]);
 	    return 1;
 	    }
-	 if (o->short_name != NULL && ! stricmp (argv[0]+1, o->short_name)
-	  || o->long_name  != NULL && ! stricmp (argv[0]+1, o->long_name))
+	 if (o->short_name && ! strcmp (argv[0]+1, o->short_name)
+	  || o->long_name  && ! strcmp (argv[0]+1, o->long_name))
             break;
          }
 
@@ -769,12 +895,12 @@ while (argc > 0)
 	 if (argv[0][0] == '-')
 	    {
 	    if (o->data_ptr && ! ignore)
-	       *((Bool *) (o->data_ptr)) = 1;
+	       *((bool *) (o->data_ptr)) = true;
 	    }
 	 else
 	    {
 	    if (o->data_ptr && ! ignore)
-	       *((Bool *) (o->data_ptr)) = 0;
+	       *((bool *) (o->data_ptr)) = false;
 	    }
 	 break;
       case OPT_CONFIRM:
@@ -799,6 +925,21 @@ while (argc > 0)
 	 if (o->data_ptr && ! ignore)
 	    *((int *) (o->data_ptr)) = atoi (argv[0]);
 	 break;
+      case OPT_WINDIM:
+         if (argc <= 1)
+	    {
+	    report_error ("missing argument after \"%s\"", argv[0]);
+	    return 1;
+	    }
+	 argv++;
+	 argc--;
+	 if (o->data_ptr && ! ignore)
+	    if (((Win_dim *) (o->data_ptr))->set (argv[0]))
+	       {
+	       report_error ("bad dimension spec \"%s\"", argv[0]);
+	       return 1;
+	       }
+	 break;
       case OPT_STRINGBUF8:
          if (argc <= 1)
             {
@@ -819,7 +960,7 @@ while (argc > 0)
 	 argv++;
 	 argc--;
 	 if (o->data_ptr && ! ignore)
-	    *((char **) (o->data_ptr)) = argv[0];
+	    *((const char **) (o->data_ptr)) = argv[0];
 	 break;
       case OPT_STRINGPTRACC:
 	 if (argc <= 1)
@@ -830,7 +971,7 @@ while (argc > 0)
 	 argv++;
 	 argc--;
 	 if (o->data_ptr && ! ignore)
-	    append_item_to_list ((char ***) o->data_ptr, argv[0]);
+	    append_item_to_list ((const char ***) o->data_ptr, argv[0]);
 	 break;
       case OPT_STRINGPTRLIST:
 	 if (argc <= 1)
@@ -843,7 +984,7 @@ while (argc > 0)
 	    argv++;
 	    argc--;
 	    if (o->data_ptr && ! ignore)
-	       append_item_to_list ((char ***) o->data_ptr, argv[0]);
+	       append_item_to_list ((const char ***) o->data_ptr, argv[0]);
 	    }
 	 break;
       default:
@@ -863,7 +1004,7 @@ return 0;
  *	dump_parameters
  *	Print a list of the parameters with their current value.
  */
-void dump_parameters (FILE *fd)
+void dump_parameters (FILE *fp)
 {
 const opt_desc_t *o;
 int desc_maxlen = 0;
@@ -873,29 +1014,39 @@ for (o = options + 1; o->opt_type != OPT_END; o++)
    {
    int len = strlen (o->desc);
    desc_maxlen = al_amax (desc_maxlen, len);
-   len = strlen (o->long_name);
-   name_maxlen = al_amax (name_maxlen, len);
+   if (o->long_name)
+      {
+      len = strlen (o->long_name);
+      name_maxlen = al_amax (name_maxlen, len);
+      }
    }
 
 for (o = options + 1; o->opt_type != OPT_END; o++)
    {
-   fprintf (fd, "%-*s  %-*s  ",
-      name_maxlen, o->long_name, desc_maxlen, o->desc);
+   if (! o->long_name)
+      continue;
+   fprintf (fp, "%-*s  %-*s  ",name_maxlen, o->long_name, desc_maxlen, o->desc);
    if (o->opt_type == OPT_BOOLEAN)
-      fprintf (fd, "%s", *((int *)o->data_ptr) ? "enabled" : "disabled");
+      fprintf (fp, "%s", *((int *)o->data_ptr) ? "enabled" : "disabled");
    else if (o->opt_type == OPT_CONFIRM)
-      fputs (confirm_i2e (*((confirm_t *) o->data_ptr)), fd);
+      fputs (confirm_i2e (*((confirm_t *) o->data_ptr)), fp);
    else if (o->opt_type == OPT_STRINGBUF8)
-      fprintf (fd, "\"%s\"", (char *) o->data_ptr);
+      fprintf (fp, "\"%s\"", (char *) o->data_ptr);
    else if (o->opt_type == OPT_STRINGPTR)
       {
       if (o->data_ptr)
-         fprintf (fd, "\"%s\"", *((char **) o->data_ptr));
+         fprintf (fp, "\"%s\"", *((char **) o->data_ptr));
       else
-         fprintf (fd, "--none--");
+         fprintf (fp, "--none--");
       }
    else if (o->opt_type == OPT_INTEGER)
-      fprintf (fd, "%d", *((int *)o->data_ptr));
+      fprintf (fp, "%d", *((int *)o->data_ptr));
+   else if (o->opt_type == OPT_WINDIM)
+      {
+      char buf[50];  // Much slack
+      ((Win_dim *) (o->data_ptr))->string (buf, sizeof buf);
+      fputs (buf, fp);
+      }
    else if (o->opt_type == OPT_STRINGPTRACC
          || o->opt_type == OPT_STRINGPTRLIST)
       {
@@ -903,14 +1054,14 @@ for (o = options + 1; o->opt_type != OPT_END; o++)
          {
          char **list;
          for (list = *((char ***) o->data_ptr); list && *list; list++)
-            fprintf (fd, "\"%s\" ", *list);
+            fprintf (fp, "\"%s\" ", *list);
          if (list == *((char ***) o->data_ptr))
-            fprintf (fd, "--none--");
+            fprintf (fp, "--none--");
          }
       else
-         fprintf (fd, "--none--");
+         fprintf (fp, "--none--");
       }
-   fputc ('\n', fd);
+   fputc ('\n', fp);
    }
 }
 
@@ -928,31 +1079,34 @@ int name_maxlen = 0;
 for (o = options + 1; o->opt_type != OPT_END; o++)
    {
    int len;
-   if (o->short_name == NULL)
+   if (! o->short_name)
       continue;
    len = strlen (o->desc);
    desc_maxlen = al_amax (desc_maxlen, len);
-   len = strlen (o->long_name);
-   name_maxlen = al_amax (name_maxlen, len);
+   if (o->long_name)
+      {
+      len = strlen (o->long_name);
+      name_maxlen = al_amax (name_maxlen, len);
+      }
    }
 
 for (o = options; o->opt_type != OPT_END; o++)
    {
-   if (o->short_name == NULL)
+   if (! o->short_name)
       continue;
 #if ! defined Y_BGI
-   if (o->flags != NULL && strchr (o->flags, 'b'))
+   if (o->flags && strchr (o->flags, 'b'))
       continue;
 #endif
 #if ! defined Y_X11
-   if (o->flags != NULL && strchr (o->flags, 'x'))
+   if (o->flags && strchr (o->flags, 'x'))
       continue;
 #endif
-   if (o->short_name != NULL)
-      fprintf (fd, " -%-2s ", o->short_name);
+   if (o->short_name)
+      fprintf (fd, " -%-3s ", o->short_name);
    else
-      fprintf (fd, "     ");
-   if (o->long_name != NULL)
+      fprintf (fd, "      ");
+   if (o->long_name)
       fprintf (fd, "-%-*s ", name_maxlen, o->long_name);
    else
       fprintf (fd, "%*s", name_maxlen + 2, "");
@@ -964,7 +1118,9 @@ for (o = options; o->opt_type != OPT_END; o++)
       case OPT_STRINGPTR:
       case OPT_STRINGPTRACC:  fprintf (fd, "<string>    "); break;
       case OPT_INTEGER:       fprintf (fd, "<integer>   "); break;
+      case OPT_WINDIM:        fprintf (fd, "<integer>[%%]"); break;
       case OPT_STRINGPTRLIST: fprintf (fd, "<string> ..."); break;
+      case OPT_END: ;  // This line is here only to silence a GCC warning.
       }
    fprintf (fd, " %s\n", o->desc);
    }
@@ -991,7 +1147,7 @@ if (external != NULL)
    if (! strcmp (external, "ask_once"))
       return YC_ASK_ONCE;
    }
-return '\0';
+return YC_ASK;
 }
 
 
@@ -1019,7 +1175,7 @@ return "?";
  *	append_item_to_list
  *	Append a string to a null-terminated string list
  */
-static void append_item_to_list (char ***list, char *item)
+static void append_item_to_list (const char ***list, const char *item)
 {
 int i;
 
@@ -1030,17 +1186,15 @@ if (*list != NULL)
    while ((*list)[i] != NULL)
       i++;
    /* expand the list */
-   *list = (char **) ResizeMemory (*list, (i + 2) * sizeof (char **));
+   *list = (const char **) ResizeMemory (*list, (i + 2) * sizeof **list);
    }
 else
    {
    /* create a new list */
-   *list = (char **) GetMemory (2 * sizeof (char **));
+   *list = (const char **) GetMemory (2 * sizeof **list);
    }
 /* append the new element */
 (*list)[i] = item;
 (*list)[i + 1] = NULL;
 }
 
-
-
