@@ -8,31 +8,30 @@
 /*
 This file is part of Yadex.
 
-Yadex incorporates code from DEU 5.21 that was put in the public
-domain in 1994 by Raphal Quinet and Brendon Wyber.
+Yadex incorporates code from DEU 5.21 that was put in the public domain in
+1994 by Raphal Quinet and Brendon Wyber.
 
-The rest of Yadex is Copyright  1997-1999 Andr Majorel.
+The rest of Yadex is Copyright  1997-2000 Andr Majorel.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
-You should have received a copy of the GNU Library General Public
-License along with this library; if not, write to the Free
-Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307, USA.
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place, Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
 #include "yadex.h"
 #include "checks.h"
 #include "dialog.h"
+#include "gamesky.h"
 #include "gfx.h"
 #include "gotoobj.h"
 #include "levels.h"
@@ -170,7 +169,8 @@ DisplayMessage (-1, -1, "Grinding...");
    display a message, then ask if the check should continue (prompt2 may be NULL)
 */
 
-Bool CheckFailed (int x0, int y0, char *prompt1, char *prompt2, Bool fatal)
+bool CheckFailed (int x0, int y0, char *prompt1, char *prompt2, bool fatal,
+    bool &first_time)
 {
 int key;
 size_t maxlen;
@@ -196,7 +196,8 @@ HideMousePointer ();
 DrawScreenBox3D (x0, y0, x0 + width - 1, y0 + height - 1);
 set_colour (LIGHTRED);
 DrawScreenText (text_x0, cur_y, "Verification failed:");
-Beep ();
+if (first_time)
+   Beep ();
 set_colour (WHITE);
 DrawScreenText (text_x0, cur_y += FONTH, prompt1);
 LogMessage ("\t%s\n", prompt1);
@@ -230,6 +231,7 @@ if (key != YK_ESC)
 //      "Verifying other objects...");
    }
 ShowMousePointer ();
+first_time = false;
 return (key == YK_ESC);
 }
 
@@ -243,6 +245,7 @@ void CheckSectors () /* SWAP! */
 int        s, n, sd;
 char huge *ends;
 char       msg1[80], msg2[80];
+bool       first_time = true;
 
 CheckingObjects ();
 LogMessage ("\nVerifying Sectors...\n");
@@ -277,7 +280,7 @@ for (s = 0; s < NumSectors; s++)
 	 {
 	 sprintf (msg1, "Sector #%d is not closed!", s);
 	 sprintf (msg2, "There is no sidedef ending at Vertex #%d", n);
-	 if (CheckFailed (-1, -1, msg1, msg2, 0))
+	 if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	    {
 	    GoToObject (OBJ_VERTICES, n);
 	    return;
@@ -287,7 +290,7 @@ for (s = 0; s < NumSectors; s++)
 	 {
 	 sprintf (msg1, "Sector #%d is not closed!", s);
 	 sprintf (msg2, "There is no sidedef starting at Vertex #%d", n);
-	 if (CheckFailed (-1, -1, msg1, msg2, 0))
+	 if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	    {
 	    GoToObject (OBJ_VERTICES, n);
 	    return;
@@ -336,7 +339,7 @@ for (n = 0; n < NumLineDefs; n++)
 	    sprintf (msg2, "Check linedef #%d (first sidedef: #%d)"
 	       " and the one facing it", n, sd);
 	    }
-	 if (CheckFailed (-1, -1, msg1, msg2, 0))
+	 if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	    {
 	    GoToObject (OBJ_LINEDEFS, n);
 	    return;
@@ -363,7 +366,7 @@ for (n = 0; n < NumLineDefs; n++)
 	    sprintf (msg2, "Check linedef #%d (second sidedef: #%d)"
 	       " and the one facing it", n, sd);
 	    }
-	 if (CheckFailed (-1, -1, msg1, msg2, 0))
+	 if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	    {
 	    GoToObject (OBJ_LINEDEFS, n);
 	    return;
@@ -383,6 +386,7 @@ void CheckCrossReferences () /* SWAP! */
 char   msg[80];
 int    n, m;
 SelPtr cur;
+bool   first_time = true;
 
 CheckingObjects ();
 LogMessage ("\nVerifying cross-references...\n");
@@ -393,7 +397,7 @@ for (n = 0; n < NumLineDefs; n++)
    if (LineDefs[n].sidedef1 < 0)
       {
       sprintf (msg, "ERROR: linedef #%d has no first sidedef!", n);
-      CheckFailed (-1, -1, msg, 0, 1);
+      CheckFailed (-1, -1, msg, 0, 1, first_time);
       GoToObject (OBJ_LINEDEFS, n);
       return;
       }
@@ -405,7 +409,7 @@ for (n = 0; n < NumLineDefs; n++)
       {
       sprintf (msg, "ERROR: linedef #%d uses the same sidedef twice (#%d)",
 	n, LineDefs[n].sidedef1);
-      CheckFailed (-1, -1, msg, 0, 1);
+      CheckFailed (-1, -1, msg, 0, 1, first_time);
       GoToObject (OBJ_LINEDEFS, n);
       return;
       }
@@ -416,7 +420,7 @@ for (n = 0; n < NumLineDefs; n++)
       {
       sprintf (msg, "ERROR: linedef #%d uses the same vertex twice (#%d)",
 	n, LineDefs[n].start);
-      CheckFailed (-1, -1, msg, 0, 1);
+      CheckFailed (-1, -1, msg, 0, 1, first_time);
       GoToObject (OBJ_LINEDEFS, n);
       return;
       }
@@ -439,8 +443,7 @@ for (n = NumLineDefs - 1; n >= 1; n--)
    }
 if (cur
   && (Expert
-      || Confirm (-1, -1,
-	   "There are multiple linedefs between the same vertices",
+   || Confirm (-1, -1, "There are multiple linedefs between the same vertices",
 	   "Do you want to delete the redundant linedefs?")))
    DeleteObjects (OBJ_LINEDEFS, &cur);
 else
@@ -451,12 +454,13 @@ CheckingObjects ();
 for (n = 0; n < NumLineDefs; n++)
    if ((LineDefs[n].flags & 0x01) == 0 && LineDefs[n].sidedef2 < 0)
       SelectObject (&cur, n);
-if (cur && (Expert || Confirm (-1, -1,
- "Some linedefs have only one side but their Im bit is not set",
- "Do you want to set the 'Impassible' flag?")))
+if (cur && (Expert
+    || Confirm (-1, -1, "Some linedefs have only one side but their I flag is"
+	" not set",
+        "Do you want to set the 'Impassible' flag?")))
    while (cur)
       {
-      LogMessage  ("Check: 1-sided linedef without Im bit: %d", cur->objnum);
+      LogMessage  ("Check: 1-sided linedef without I flag: %d", cur->objnum);
       LineDefs[cur->objnum].flags |= 0x01;
       UnSelectObject (&cur, cur->objnum);
       }
@@ -469,8 +473,8 @@ for (n = 0; n < NumLineDefs; n++)
       SelectObject (&cur, n);
 if (cur
   && (Expert
-      || Confirm (-1, -1,
-	   "Some linedefs have only one side but their 2S bit is set",
+      || Confirm (-1, -1, "Some linedefs have only one side but their 2 flag"
+	  " is set",
 	   "Do you want to clear the 'two-sided' flag?")))
    {
    while (cur)
@@ -593,46 +597,53 @@ int  n;
 int  sd1, sd2;
 int  s1, s2;
 char msg1[80], msg2[80];
+bool first_time = true;
 
 CheckingObjects ();
 LogMessage ("\nVerifying textures...\n");
 ObjectsNeeded (OBJ_SECTORS, 0);
 for (n = 0; n < NumSectors; n++)
    {
-   if (Sectors[n].ceilt[0] == '-' && Sectors[n].ceilt[1] == '\0')
+   if (strcmp (Sectors[n].ceilt, "-") == 0
+     || strcmp (Sectors[n].ceilt, "") == 0
+     || memcmp (Sectors[n].ceilt, "        ", 8) == 0)
       {
       sprintf (msg1, "Error: sector #%d has no ceiling texture", n);
       sprintf (msg2, "You probably used a brain-damaged editor to do that...");
-      CheckFailed (-1, -1, msg1, msg2, 1);
+      CheckFailed (-1, -1, msg1, msg2, 1, first_time);
       GoToObject (OBJ_SECTORS, n);
       return;
       }
-   if (Sectors[n].floort[0] == '-' && Sectors[n].floort[1] == '\0')
+   if (strcmp (Sectors[n].floort, "-") == 0
+     || strcmp (Sectors[n].floort, "") == 0
+     || memcmp (Sectors[n].floort, "        ", 8) == 0)
       {
       sprintf (msg1, "Error: sector #%d has no floor texture", n);
       sprintf (msg2, "You probably used a brain-damaged editor to do that...");
-      CheckFailed (-1, -1, msg1, msg2, 1);
+      CheckFailed (-1, -1, msg1, msg2, 1, first_time);
       GoToObject (OBJ_SECTORS, n);
       return;
       }
    if (Sectors[n].ceilh < Sectors[n].floorh)
       {
       sprintf (msg1,
-	"Error: Sector #%d has its ceiling lower than its floor", n);
+	"Error: sector #%d has its ceiling lower than its floor", n);
       sprintf (msg2,
 	"The textures will never be displayed if you cannot go there");
-      CheckFailed (-1, -1, msg1, msg2, 1);
+      CheckFailed (-1, -1, msg1, msg2, 1, first_time);
       GoToObject (OBJ_SECTORS, n);
       return;
       }
+#if 0  /* AYM 2000-08-13 */
    if (Sectors[n].ceilh - Sectors[n].floorh > 1023)
       {
       sprintf (msg1, "Error: sector #%d has its ceiling too high", n);
       sprintf (msg2, "The maximum difference allowed is 1023 (ceiling - floor)");
-      CheckFailed (-1, -1, msg1, msg2, 1);
+      CheckFailed (-1, -1, msg1, msg2, 1, first_time);
       GoToObject (OBJ_SECTORS, n);
       return;
       }
+#endif
    }
 
 for (n = 0; n < NumLineDefs; n++)
@@ -657,12 +668,12 @@ for (n = 0; n < NumLineDefs; n++)
 	   " sidedef #%d has no middle texture", n, sd1);
 	 sprintf (msg2, "Do you want to set the texture to \"%s\""
 	   " and continue?", default_middle_texture);
-	 if (CheckFailed (-1, -1, msg1, msg2, 0))
+	 if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	    {
 	    GoToObject (OBJ_LINEDEFS, n);
 	    return;
 	    }
-	 strncpy (SideDefs[sd1].tex3, default_middle_texture, 8);
+	 strncpy (SideDefs[sd1].tex3, default_middle_texture, WAD_TEX_NAME);
          MadeChanges = 1;
 	 CheckingObjects ();
 	 }
@@ -670,19 +681,18 @@ for (n = 0; n < NumLineDefs; n++)
    if (is_obj (s1) && is_obj (s2) && Sectors[s1].ceilh > Sectors[s2].ceilh)
       {
       if (SideDefs[sd1].tex1[0] == '-' && SideDefs[sd1].tex1[1] == '\0'
-	  && (strncmp (Sectors[s1].ceilt, "F_SKY1", 8)
-	   || strncmp (Sectors[s2].ceilt, "F_SKY1", 8)))
+	  && (! is_sky (Sectors[s1].ceilt) || ! is_sky (Sectors[s2].ceilt)))
 	 {
 	 sprintf (msg1, "Error in first sidedef of linedef #%d:"
 	   " sidedef #%d has no upper texture", n, sd1);
 	 sprintf (msg2, "Do you want to set the texture to \"%s\""
 	   " and continue?", default_upper_texture);
-	 if (CheckFailed (-1, -1, msg1, msg2, 0))
+	 if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	    {
 	    GoToObject (OBJ_LINEDEFS, n);
 	    return;
 	    }
-	 strncpy (SideDefs[sd1].tex1, default_upper_texture, 8);
+	 strncpy (SideDefs[sd1].tex1, default_upper_texture, WAD_TEX_NAME);
          MadeChanges = 1;
 	 CheckingObjects ();
 	 }
@@ -695,12 +705,12 @@ for (n = 0; n < NumLineDefs; n++)
 	   " sidedef #%d has no lower texture", n, sd1);
 	 sprintf (msg2, "Do you want to set the texture to \"%s\""
 	   " and continue?", default_lower_texture);
-	 if (CheckFailed (-1, -1, msg1, msg2, 0))
+	 if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	    {
 	    GoToObject (OBJ_LINEDEFS, n);
 	    return;
 	    }
-	 strncpy (SideDefs[sd1].tex2, default_lower_texture, 8);
+	 strncpy (SideDefs[sd1].tex2, default_lower_texture, WAD_TEX_NAME);
          MadeChanges = 1;
 	 CheckingObjects ();
 	 }
@@ -708,19 +718,18 @@ for (n = 0; n < NumLineDefs; n++)
    if (is_obj (s1) && is_obj (s2) && Sectors[s2].ceilh > Sectors[s1].ceilh)
       {
       if (SideDefs[sd2].tex1[0] == '-' && SideDefs[sd2].tex1[1] == '\0'
-	  && (strncmp (Sectors[s1].ceilt, "F_SKY1", 8)
-	   || strncmp (Sectors[s2].ceilt, "F_SKY1", 8)))
+	  && (! is_sky (Sectors[s1].ceilt) || ! is_sky (Sectors[s2].ceilt)))
 	 {
 	 sprintf (msg1, "Error in second sidedef of linedef #%d:"
 	   " sidedef #%d has no upper texture", n, sd2);
 	 sprintf (msg2, "Do you want to set the texture to \"%s\""
 	   " and continue?", default_upper_texture);
-	 if (CheckFailed (-1, -1, msg1, msg2, 0))
+	 if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	    {
 	    GoToObject (OBJ_LINEDEFS, n);
 	    return;
 	    }
-	 strncpy (SideDefs[sd2].tex1, default_upper_texture, 8);
+	 strncpy (SideDefs[sd2].tex1, default_upper_texture, WAD_TEX_NAME);
          MadeChanges = 1;
 	 CheckingObjects ();
 	 }
@@ -733,12 +742,12 @@ for (n = 0; n < NumLineDefs; n++)
 	   " sidedef #%d has no lower texture", n, sd2);
 	 sprintf (msg2, "Do you want to set the texture to \"%s\""
 	   " and continue?", default_lower_texture);
-	 if (CheckFailed (-1, -1, msg1, msg2, 0))
+	 if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	    {
 	    GoToObject (OBJ_LINEDEFS, n);
 	    return;
 	    }
-	 strncpy (SideDefs[sd2].tex2, default_lower_texture, 8);
+	 strncpy (SideDefs[sd2].tex2, default_lower_texture, WAD_TEX_NAME);
          MadeChanges = 1;
 	 CheckingObjects ();
 	 }
@@ -751,14 +760,14 @@ for (n = 0; n < NumLineDefs; n++)
    check if a texture name matches one of the elements of a list
 */
 
-Bool IsTextureNameInList (char *name, char **list, int numelems)
+bool IsTextureNameInList (char *name, char **list, int numelems)
 {
 int n;
 
 for (n = 0; n < numelems; n++)
-   if (! strnicmp (name, list[n], 8))
-      return 1;
-return 0;
+   if (! y_strnicmp (name, list[n], WAD_TEX_NAME))
+      return true;
+return false;
 }
 
 
@@ -770,32 +779,36 @@ void CheckTextureNames () /* SWAP! */
 {
 int  n;
 char msg1[80], msg2[80];
+bool first_time = true;
 
 CheckingObjects ();
 LogMessage ("\nVerifying texture names...\n");
+
+// AYM 2000-07-24: could someone explain this one ?
 if (! FindMasterDir (MasterDir, "F2_START"))
    NumThings--;
+
 ObjectsNeeded (OBJ_SECTORS, 0);
 for (n = 0; n < NumSectors; n++)
    {
-   if (! IsTextureNameInList (Sectors[n].ceilt, FTexture, NumFTexture))
+   if (! is_flat_name_in_list (Sectors[n].ceilt))
       {
       sprintf (msg1, "Invalid ceiling texture in sector #%d", n);
-      sprintf (msg2, "The name \"%.8s\" is not a floor/ceiling texture",
-	Sectors[n].ceilt);
-      if (CheckFailed (-1, -1, msg1, msg2, 0))
+      sprintf (msg2, "The name \"%.*s\" is not a floor/ceiling texture",
+	(int) WAD_FLAT_NAME, Sectors[n].ceilt);
+      if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	 {
 	 GoToObject (OBJ_SECTORS, n);
 	 return;
 	 }
       CheckingObjects ();
       }
-   if (! IsTextureNameInList (Sectors[n].floort, FTexture, NumFTexture))
+   if (! is_flat_name_in_list (Sectors[n].floort))
       {
       sprintf (msg1, "Invalid floor texture in sector #%d", n);
-      sprintf (msg2, "The name \"%.8s\" is not a floor/ceiling texture",
-	Sectors[n].floort);
-      if (CheckFailed (-1, -1, msg1, msg2, 0))
+      sprintf (msg2, "The name \"%.*s\" is not a floor/ceiling texture",
+	(int) WAD_FLAT_NAME, Sectors[n].floort);
+      if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	 {
 	 GoToObject (OBJ_SECTORS, n);
 	 return;
@@ -809,9 +822,9 @@ for (n = 0; n < NumSideDefs; n++)
    if (! IsTextureNameInList (SideDefs[n].tex1, WTexture, NumWTexture))
       {
       sprintf (msg1, "Invalid upper texture in sidedef #%d", n);
-      sprintf (msg2, "The name \"%.8s\" is not a wall texture",
-	SideDefs[n].tex1);
-      if (CheckFailed (-1, -1, msg1, msg2, 0))
+      sprintf (msg2, "The name \"%.*s\" is not a wall texture",
+	(int) WAD_TEX_NAME, SideDefs[n].tex1);
+      if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	 {
 	 GoToObject (OBJ_SIDEDEFS, n);
 	 return;
@@ -821,9 +834,9 @@ for (n = 0; n < NumSideDefs; n++)
    if (! IsTextureNameInList (SideDefs[n].tex2, WTexture, NumWTexture))
       {
       sprintf (msg1, "Invalid lower texture in sidedef #%d", n);
-      sprintf (msg2, "The name \"%.8s\" is not a wall texture",
-	SideDefs[n].tex2);
-      if (CheckFailed (-1, -1, msg1, msg2, 0))
+      sprintf (msg2, "The name \"%.*s\" is not a wall texture",
+	(int) WAD_TEX_NAME, SideDefs[n].tex2);
+      if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	 {
 	 GoToObject (OBJ_SIDEDEFS, n);
 	 return;
@@ -833,9 +846,9 @@ for (n = 0; n < NumSideDefs; n++)
    if (! IsTextureNameInList (SideDefs[n].tex3, WTexture, NumWTexture))
       {
       sprintf (msg1, "Invalid middle texture in sidedef #%d", n);
-      sprintf (msg2, "The name \"%.8s\" is not a wall texture",
-	SideDefs[n].tex3);
-      if (CheckFailed (-1, -1, msg1, msg2, 0))
+      sprintf (msg2, "The name \"%.*s\" is not a wall texture",
+	(int) WAD_TEX_NAME, SideDefs[n].tex3);
+      if (CheckFailed (-1, -1, msg1, msg2, 0, first_time))
 	 {
 	 GoToObject (OBJ_SIDEDEFS, n);
 	 return;
@@ -850,41 +863,41 @@ for (n = 0; n < NumSideDefs; n++)
    check for players starting points
 */
 
-Bool CheckStartingPos () /* SWAP! */
+bool CheckStartingPos () /* SWAP! */
 {
 char msg1[80], msg2[80];
-Bool p1 = 0;
-Bool p2 = 0;
-Bool p3 = 0;
-Bool p4 = 0;
-int  dm = 0;
+bool p1 = false;
+bool p2 = false;
+bool p3 = false;
+bool p4 = false;
+size_t dm = 0;
 int  t;
 
 ObjectsNeeded (OBJ_THINGS, 0);
 for (t = 0; t < NumThings; t++)
    {
    if (Things[t].type == THING_PLAYER1)
-      p1 = 1;
+      p1 = true;
    if (Things[t].type == THING_PLAYER2)
-      p2 = 1;
+      p2 = true;
    if (Things[t].type == THING_PLAYER3)
-      p3 = 1;
+      p3 = true;
    if (Things[t].type == THING_PLAYER4)
-      p4 = 1;
+      p4 = true;
    if (Things[t].type == THING_DEATHMATCH)
       dm++;
    }
 if (! p1)
    {
    Beep ();
-   sprintf (msg1,
-     "Warning: there is no player 1 starting point!  Doom will crash if");
-   sprintf (msg2,
-     "you play with this level.  Do you really want to save it?");
-   return Confirm (-1, -1, msg1, msg2);
+   if (! Confirm (-1, -1, "Warning: there is no player 1 starting point. The"
+       " game", "will crash if you play with this level. Save anyway ?"))
+      return false;
+   else 
+      return true;  // No point in doing further checking !
    }
 if (Expert)
-   return 1;
+   return true;
 if (! p2 || ! p3 || ! p4)
    {
    if (! p4)
@@ -893,28 +906,32 @@ if (! p2 || ! p3 || ! p4)
      t = 3;
    if (! p2)
      t = 2;
-   sprintf (msg1, "Warning: there is no player %d starting point."
-     "  You will not be able to", t);
-   sprintf (msg2, "use this level for multi-player games."
-     "  Press Y to return to the editor.");
-   return !Confirm (-1, -1, msg1, msg2);
+   sprintf (msg1, "Warning: there is no player %d start."
+     " You will not be able", t);
+   sprintf (msg2, "to use this level for multi-player games."
+     " Save anyway ?");
+   if (! Confirm (-1, -1, msg1, msg2))
+      return false;
+   else
+      return true;  // No point in doing further checking !
    }
 if (dm < DOOM_MIN_DEATHMATCH_STARTS)
    {
    if (dm == 0)
-     sprintf (msg1, "Warning: there are no deathmatch starting points."
-       "  You need at least %d", DOOM_MIN_DEATHMATCH_STARTS);
+     sprintf (msg1, "Warning: there are no deathmatch starts."
+       " You need at least %d", DOOM_MIN_DEATHMATCH_STARTS);
    else if (dm == 1)
-     sprintf (msg1, "Warning: there is only one deathmatch starting point."
-       "  You need at least %d", DOOM_MIN_DEATHMATCH_STARTS);
+     sprintf (msg1, "Warning: there is only one deathmatch start."
+       " You need at least %d", DOOM_MIN_DEATHMATCH_STARTS);
    else
-     sprintf (msg1, "Warning: there are only %d deathmatch starting points."
-       "  You need at least %d", dm, DOOM_MIN_DEATHMATCH_STARTS);
-   sprintf (msg2, "starting points to play deathmatch games."
-     "  Press Y to return to the editor.");
-   return !Confirm (-1, -1, msg1, msg2);
+     sprintf (msg1, "Warning: there are only %d deathmatch starts."
+       " You need at least %d", dm, DOOM_MIN_DEATHMATCH_STARTS);
+   sprintf (msg2, "deathmatch starts to play deathmatch games."
+     " Save anyway ?");
+   if (! Confirm (-1, -1, msg1, msg2))
+     return false;
    }
-return 1;
+return true;
 }
 
 
