@@ -5,7 +5,7 @@
  */
 
 
-Bool Confirm (int, int, const char *, const char *);
+bool Confirm (int, int, const char *, const char *);
 int Confirm2 (int x0, int y0, confirm_t *confirm_flag,
    const char *prompt1, const char *prompt2);
 void Notify (int, int, const char *, const char *);
