@@ -8,25 +8,23 @@
 /*
 This file is part of Yadex.
 
-Yadex incorporates code from DEU 5.21 that was put in the public
-domain in 1994 by Raphal Quinet and Brendon Wyber.
+Yadex incorporates code from DEU 5.21 that was put in the public domain in
+1994 by Raphal Quinet and Brendon Wyber.
 
-The rest of Yadex is Copyright  1997-1999 Andr Majorel.
+The rest of Yadex is Copyright  1997-2000 Andr Majorel.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
-You should have received a copy of the GNU Library General Public
-License along with this library; if not, write to the Free
-Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307, USA.
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place, Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
@@ -40,8 +38,10 @@ Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 #include "gfx.h"
 #include "levels.h"
 #include "oldmenus.h"
+#include "s_swapf.h"
 #include "selectn.h"
 #include "t_spin.h"
+#include "x_mirror.h"
 #include "x_rotate.h"
 
 
@@ -79,7 +79,7 @@ return val;
    ask for an object number and display a warning message
 */
 
-int InputObjectXRef (int x0, int y0, int objtype, Bool allownone, int curobj)
+int InputObjectXRef (int x0, int y0, int objtype, bool allownone, int curobj)
 {
 int val, key;
 char prompt[80];
@@ -119,7 +119,7 @@ return val;
    ask for two vertex numbers and check for maximum valid number
 */
 
-Bool Input2VertexNumbers (int x0, int y0, const char *prompt1, int *v1, int *v2)
+bool Input2VertexNumbers (int x0, int y0, const char *prompt1, int *v1, int *v2)
 {
 int  key;
 int  maxlen, first;
@@ -154,7 +154,7 @@ entry2_y0 = text_y0 + 5 * FONTH - HOLLOW_BORDER - NARROW_VSPACING;
 
 DrawScreenBox3D (x0, y0,
    x0 + 2 * BOX_BORDER + 2 * WIDE_HSPACING
-      + max (entry_width + 13 * FONTW, maxlen * FONTW) - 1,
+      + y_max (entry_width + 13 * FONTW, maxlen * FONTW) - 1,
    y0 + 2 * BOX_BORDER + 2 * WIDE_VSPACING + 6 * FONTH - 1);
 set_colour (WHITE);
 DrawScreenText (text_x0, text_y0, prompt1);
@@ -243,8 +243,8 @@ switch (objtype)
 	 {
 	 case 1:
 	    val = InputIntegerValue (x0 + 42, subwin_y0,
-               min (MapMinX, -10000),
-               max (MapMaxX, 10000),
+               y_min (MapMinX, -10000),
+               y_max (MapMaxX, 10000),
                Vertices[obj->objnum].x);
 	    if (val != IIV_CANCEL)
 	       {
@@ -258,8 +258,8 @@ switch (objtype)
 
 	 case 2:
 	    val = InputIntegerValue (x0 + 42, subwin_y0,
-               min (MapMinY, -10000),
-               max (MapMaxY, 10000),
+               y_min (MapMinY, -10000),
+               y_max (MapMaxY, 10000),
                Vertices[obj->objnum].y);
 	    if (val != IIV_CANCEL)
 	       {
@@ -288,12 +288,12 @@ switch (objtype)
    Yuck!  Dirty piece of code...
 */
 
-Bool Input2Numbers (int x0, int y0, const char *name1, const char *name2,
+bool Input2Numbers (int x0, int y0, const char *name1, const char *name2,
    int v1max, int v2max, int *v1, int *v2)
 {
 int  key;
 int  maxlen, first;
-Bool ok;
+bool ok;
 char prompt[80];
 // FIXME copied from InputInteger()...
 int  entry_width = 2 * HOLLOW_BORDER + 2 * NARROW_HSPACING + 7 * FONTW;
@@ -312,12 +312,8 @@ int  entry2_text_x0;
 int  entry2_text_y0;
 
 HideMousePointer ();
-#ifdef Y_SNPRINTF
-snprintf (prompt, sizeof prompt, "Give the %s and %s for the object:",
+y_snprintf (prompt, sizeof prompt, "Give the %s and %s for the object:",
    name1, name2);
-#else
-sprintf (prompt, "Give the %s and %s for the object:", name1, name2);
-#endif
 maxlen = strlen (prompt);
 if (x0 < 0)
    x0 = (ScrMaxX - 25 - FONTW * maxlen) / 2;
@@ -350,13 +346,13 @@ first = 1;
 key = 0;
 for (;;)
    {
-   ok = 1;
+   ok = true;
    DrawScreenBoxHollow (entry1_out_x0, entry1_out_y0,
       entry1_out_x1, entry1_out_y1, BLACK);
    if (*v1 < 0 || *v1 > v1max)
       {
       set_colour (DARKGRAY);
-      ok = 0;
+      ok = false;
       }
    else
       set_colour (LIGHTGRAY);
@@ -366,7 +362,7 @@ for (;;)
    if (*v2 < 0 || *v2 > v2max)
       {
       set_colour (DARKGRAY);
-      ok = 0;
+      ok = false;
       }
    else
       set_colour (LIGHTGRAY);
@@ -587,10 +583,12 @@ if (val > 1 && ! *list)
 switch (val)
    {
    case 1:
+      // * -> First free tag number
       sprintf (msg, "First free tag number: %d", FindFreeTag ());
       Notify (-1, -1, msg, 0);
       break;
    case 2:
+      // * -> Rotate and scale
       if ((objtype == OBJ_THINGS
          || objtype == OBJ_VERTICES) && ! (*list)->next)
 	 {
@@ -608,15 +606,12 @@ switch (val)
             (double) scale * 0.01);
       break;
    case 3:
-      if (objtype == OBJ_VERTICES)
-	 {
-	 DeleteVerticesJoinLineDefs (*list);
-	 ForgetSelection (list);
-	 }
-      else if (objtype == OBJ_LINEDEFS)
+      // Linedef -> Split
+      if (objtype == OBJ_LINEDEFS)
 	 {
 	 SplitLineDefs (*list);
 	 }
+      // Sector -> Make door from sector
       else if (objtype == OBJ_SECTORS)
 	 {
 	 if ((*list)->next)
@@ -629,15 +624,21 @@ switch (val)
 	    MakeDoorFromSector ((*list)->objnum);
 	    }
 	 }
+      // Thing -> Spin 45 clockwise
       else if (objtype == OBJ_THINGS)
+	 {
          spin_things (*list, -45);
+	 }
+      // Vertex -> Delete and join linedefs
+      else if (objtype == OBJ_VERTICES)
+	 {
+	 DeleteVerticesJoinLineDefs (*list);
+	 ForgetSelection (list);
+	 }
       break;
    case 4:
-      if (objtype == OBJ_VERTICES)
-	 {
-	 MergeVertices (list);
-	 }
-      else if (objtype == OBJ_LINEDEFS)
+      // Linedef -> Split linedefs and sector
+      if (objtype == OBJ_LINEDEFS)
 	 {
 	 if (! (*list)->next || (*list)->next->next)
 	    {
@@ -650,6 +651,7 @@ switch (val)
 	    ForgetSelection (list);
 	    }
 	 }
+      // Sector -> Make lift from sector
       else if (objtype == OBJ_SECTORS)
 	 {
 	 if ((*list)->next)
@@ -662,27 +664,22 @@ switch (val)
 	    MakeLiftFromSector ((*list)->objnum);
 	    }
 	 }
+      // Thing -> Spin 45 counter-clockwise
       else if (objtype == OBJ_THINGS)
          spin_things (*list, 45);
+      // Vertex -> Merge
+      else if (objtype == OBJ_VERTICES)
+	 {
+	 MergeVertices (list);
+	 }
       break;
    case 5:
-      if (objtype == OBJ_VERTICES)
-	 {
-	 if (! (*list)->next || (*list)->next->next)
-	    {
-	    Beep ();
-	    Notify (-1, -1, "You must select exactly two Vertices", 0);
-	    }
-	 else
-	    {
-	    SplitSector ((*list)->objnum, (*list)->next->objnum);
-	    ForgetSelection (list);
-	    }
-	 }
-      else if (objtype == OBJ_LINEDEFS)
+      // Linedef -> Delete linedefs and join sectors
+      if (objtype == OBJ_LINEDEFS)
 	 {
 	 DeleteLineDefsJoinSectors (list);
 	 }
+      // Sector -> Distribute sector floor heights
       else if (objtype == OBJ_SECTORS)
 	 {
 	 if (! (*list)->next || ! (*list)->next->next)
@@ -695,12 +692,33 @@ switch (val)
 	    DistributeSectorFloors (*list);
 	    }
 	 }
+      // Thing -> Mirror horizontally
+      else if (objtype == OBJ_THINGS)
+         {
+	 flip_mirror (*list, OBJ_THINGS, 'm');
+	 }
+      // Vertex -> Add linedef and split sector
+      else if (objtype == OBJ_VERTICES)
+	 {
+	 if (! (*list)->next || (*list)->next->next)
+	    {
+	    Beep ();
+	    Notify (-1, -1, "You must select exactly two vertices", 0);
+	    }
+	 else
+	    {
+	    SplitSector ((*list)->next->objnum, (*list)->objnum);
+	    ForgetSelection (list);
+	    }
+	 }
       break;
    case 6:
+      // Linedef -> Flip
       if (objtype == OBJ_LINEDEFS)
 	 {
 	 FlipLineDefs (*list, 1);
 	 }
+      // Sector -> Distribute ceiling heights
       else if (objtype == OBJ_SECTORS)
 	 {
 	 if (! (*list)->next || ! (*list)->next->next)
@@ -713,8 +731,19 @@ switch (val)
 	    DistributeSectorCeilings (*list);
 	    }
 	 }
+      // Things -> Mirror vertically
+      else if (objtype == OBJ_THINGS)
+	 {
+	 flip_mirror (*list, OBJ_THINGS, 'f');
+         }
+      // Vertex -> Mirror horizontally
+      else if (objtype == OBJ_VERTICES)
+	 {
+	 flip_mirror (*list, OBJ_VERTICES, 'm');
+	 }
       break;
    case 7:
+      // Linedefs -> Swap sidedefs
       if (objtype == OBJ_LINEDEFS)
 	 {
 	 if (Expert
@@ -723,15 +752,19 @@ switch (val)
                "You may get strange results if you don't know what you are doing..."))
 	    FlipLineDefs (*list, 0);
 	 }
+      // Sectors -> Raise or lower
       else if (objtype == OBJ_SECTORS)
-	RaiseOrLowerSectors (*list);
+	 {
+	 RaiseOrLowerSectors (*list);
+	 }
+      // Vertices -> Mirror vertically
+      else if (objtype == OBJ_VERTICES)
+	 {
+	 flip_mirror (*list, OBJ_VERTICES, 'f');
+	 }
       break;
    case 8:
-      if (objtype == OBJ_SECTORS)
-	 {
-	 BrightenOrDarkenSectors (*list);
-	 break;
-	 }
+      // Linedef ->  Align textures vertically
       if (objtype == OBJ_LINEDEFS)
 	 {
 	 SelPtr sdlist, cur;
@@ -749,13 +782,14 @@ switch (val)
 	 /* align the textures along the Y axis (height) */
 	 AlignTexturesY (&sdlist);
 	 }
-      /* AYM 19971219 : is the fall through intentional ? */
-   case 9:
-      if (objtype == OBJ_SECTORS)
+      // Sector -> Brighten or darken
+      else if (objtype == OBJ_SECTORS)
 	 {
-	 NotImplemented ();  // FIXME
-	 break;
+	 BrightenOrDarkenSectors (*list);
 	 }
+      break;
+   case 9:
+      // Linedef -> Align texture horizontally
       if (objtype == OBJ_LINEDEFS)
 	 {
 	 SelPtr sdlist, cur;
@@ -773,6 +807,12 @@ switch (val)
 	 /* align the textures along the X axis (width) */
 	 AlignTexturesX (&sdlist);
 	 }
+      // Sector -> Unlink room
+      else if (objtype == OBJ_SECTORS)
+	 {
+	 NotImplemented ();  // FIXME
+	 break;
+	 }
       break;
 
    case 10:
@@ -791,18 +831,31 @@ switch (val)
                cross-references check will delete it anyway. */
 	    }
 	 }
+      // Sector -> Mirror horizontally
+      else if (objtype == OBJ_SECTORS)
+	 {
+	 flip_mirror (*list, OBJ_SECTORS, 'm');
+	 }
       break;
 
    case 11:
       // Linedef -> Make rectangular nook
       if (objtype == OBJ_LINEDEFS)
 	 MakeRectangularNook (*list, 32, 16, 0);
+      // Sector -> Mirror vertically
+      else if (objtype == OBJ_SECTORS)
+	 {
+	 flip_mirror (*list, OBJ_SECTORS, 'f');
+	 }
       break;
 
    case 12:
       // Linedef -> Make rectangular boss
       if (objtype == OBJ_LINEDEFS)
 	 MakeRectangularNook (*list, 32, 16, 1);
+      // Sector -> Swap flats
+      else if (objtype == OBJ_SECTORS)
+	 swap_flats (*list);
       break;
 
    case 13:
@@ -839,6 +892,16 @@ switch (val)
          unlink_sidedef (*list, 0, 1);
       break;
 
+   case 17:
+      // Linedef -> Mirror horizontally
+      flip_mirror (*list, OBJ_LINEDEFS, 'm');
+      break;
+      
+   case 18 :
+      // Linedef -> Mirror vertically
+      flip_mirror (*list, OBJ_LINEDEFS, 'f');
+      break;
+
    }
 }
 
