@@ -8,45 +8,44 @@
 /*
 This file is part of Yadex.
 
-Yadex incorporates code from DEU 5.21 that was put in the public
-domain in 1994 by Raphal Quinet and Brendon Wyber.
+Yadex incorporates code from DEU 5.21 that was put in the public domain in
+1994 by Raphal Quinet and Brendon Wyber.
 
-The rest of Yadex is Copyright  1997-1999 Andr Majorel.
+The rest of Yadex is Copyright  1997-2000 Andr Majorel.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
-You should have received a copy of the GNU Library General Public
-License along with this library; if not, write to the Free
-Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307, USA.
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place, Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
 #include "yadex.h"
 #include <math.h>
-#include <errno.h>
+#ifdef Y_X11
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#endif
 #include "acolours.h"
 #include "gcolour1.h"
 #include "gcolour2.h"
+#include "gcolour3.h"
 #include "gfx.h"
 #include "levels.h"  // Level
+#include "x11.h"
 
 #ifdef Y_DOS
 #include <direct.h>
 #endif
 
-#ifdef Y_X11
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#endif
 
 
 /* if your graphics driver doesn't like circles, draw squares instead */
@@ -59,11 +58,11 @@ Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 
 /* Parameters set by the command line args and config file */
 const char *BGIDriver = "VESA";	// BGI: default extended BGI driver
-Bool  CirrusCursor = 0;		// use HW cursor on Cirrus Logic VGA cards
-Bool  FakeCursor = 0;		// use a "fake" mouse cursor
+bool  CirrusCursor = false;	// use HW cursor on Cirrus Logic VGA cards
+bool  FakeCursor = false;	// use a "fake" mouse cursor
 const char *font_name = NULL;	// X: the name of the font to load
-int   initial_window_width =640;// X: the name says it all
-int   initial_window_height=480;// X: the name says it all
+Win_dim initial_window_width ("90%");	// X: the name says it all
+Win_dim initial_window_height ("90%");	// X: the name says it all
 int   no_pixmap;		// X: use no pixmap -- direct window output
 int   VideoMode = 2;		// BGI: default video mode for VESA cards
 
@@ -71,13 +70,13 @@ int   VideoMode = 2;		// BGI: default video mode for VESA cards
 int GfxMode = 0;	// graphics mode number, or 0 for text
 			// 1 = 320x200, 2 = 640x480, 3 = 800x600, 4 = 1kx768
 			// positive = 16 colors, negative = 256 colors
-int OrigX;		// the X origin
-int OrigY;		// the Y origin
+int OrigX;		// Map X-coord of centre of screen/window
+int OrigY;		// Map Y-coord of centre of screen/window
 float Scale;		// the scale value
-int ScrMaxX;		// maximum X screen coord
-int ScrMaxY;		// maximum Y screen coord
-int ScrCenterX;		// X coord of screen center
-int ScrCenterY;		// Y coord of screen center
+int ScrMaxX;		// Maximum display X-coord of screen/window
+int ScrMaxY;		// Maximum display Y-coord of screen/window
+int ScrCenterX;		// Display X-coord of centre of screen/window
+int ScrCenterY;		// Display Y-coord of centre of screen/window
 int FONTH;
 int FONTW;
 int font_xofs;
@@ -88,14 +87,6 @@ int font_yofs;
 Display *dpy;		// The X display
 int      scn;		// The X screen number
 Colormap cmap = 0;	// The X colormap
-#ifdef MANY_GC
-			// The GCs for the application colours.
-			// The first NCOLOURS have the "copy" function.
-			// The following NCOLOURS have the "xor" function.
-			// The 2 x NCOLOURS following are the same as the
-			// above except that line is thick instead of thin.
-GC       std_gc[2][2][NCOLOURS];
-#endif MANY_GC
 GC       gc;		// Default GC as set by set_colour(), SetDrawingMode()
                         // and SetLineThickness()
 GC       pixmap_gc;	// The GC used to clear the pixmap
@@ -122,6 +113,9 @@ int      win_vis_class;	// The class of win's visual
 VisualID win_vis_id;	// The ID of win's visual
 int      win_depth;	// The depth of win in bits
 int      win_bpp;	// The depth of win in bytes
+int      x_server_big_endian = 0;	// Is the X server big endian ?
+int      ximage_bpp;	// Number of bytes per pixels in XImages
+int      ximage_quantum;// Pad XImages lines to a multiple of that many bytes
 static pcolour_t *app_colour = 0;	// Pixel values for the app. colours
 static int DrawingMode    = 0;		// 0 = copy, 1 = xor
 static int LineThickness  = 0;		// 0 = thin, 1 = thick
@@ -130,6 +124,7 @@ int      text_dot         = 0;		// DrawScreenText() debug flag
 static acolour_t colour_stack[4];
 static int       colour_stack_pointer = 0;
 static Font      font_xfont;
+static bool      default_font = true;
 
 #if defined Y_BGI && defined CIRRUS_PATCH
 char mp[256];
@@ -171,22 +166,27 @@ char HWCursor[] =
 #endif /* Y_BGI && CIRRUS_PATCH */
 
 
+/*
+ *	Prototypes
+ */
 #ifdef Y_BGI
-int cooked_installuserdriver (const char far *__name,
+static int cooked_installuserdriver (const char far *__name,
                              int huge (*detect)(void));
 #endif
 
 /*
-   initialize the graphics display
-*/
-
-void InitGfx (void)
+ *	InitGfx - initialize the graphics display
+ *
+ *	Return 0 on success, non-zero on failure
+ */
+int InitGfx (void)
 {
-int width = initial_window_width;
-int height = initial_window_height;
+// Initialization is in fact not necessary
+int width;
+int height;
 
 #if defined Y_BGI
-static Bool firsttime = 1;
+static bool firsttime = true;
 static int  gdriver;
 static int  gmode;
 int         errorcode = grNoInitGraph;
@@ -229,7 +229,7 @@ verbmsg ("GfxMode=%d\n", GfxMode);
 setlinestyle (0, 0, 1);
 setbkcolor (TranslateToDoomColor (BLACK));
 settextstyle (0, 0, 1);
-firsttime = 0;
+firsttime = false;
 width = getmaxx () + 1;
 height = getmaxy () + 1;
 
@@ -240,20 +240,49 @@ height = getmaxy () + 1;
  */
 dpy = XOpenDisplay (0);
 if (! dpy)
-   fatal_error ("Can't open display");
+   {
+   report_error ("Can't open display");
+   return 1;
+   }
 scn = DefaultScreen (dpy);
+{
+verbmsg ("X server endianness: ");
+int r = ImageByteOrder (dpy);
+if (r == LSBFirst)
+   {
+   verbmsg ("little-endian\n");
+   x_server_big_endian = 0;
+   }
+else if (r == MSBFirst)
+   {
+   verbmsg ("big-endian\n");
+   x_server_big_endian = 1;
+   }
+else
+   {
+   verbmsg ("unknown\n");
+   warn ("don't understand X server's endianness code %d\n", r);
+   warn ("assuming same endianness as CPU.\n");
+   x_server_big_endian = cpu_big_endian;
+   }
+}
+int screen_width  = DisplayWidth  (dpy, scn);
+int screen_height = DisplayHeight (dpy, scn);
+
 
 /*
  *	Create the window
  */
+width  = initial_window_width.pixels  (screen_width);
+height = initial_window_height.pixels (screen_height);
 win = XCreateSimpleWindow (dpy, DefaultRootWindow (dpy),
    10, 10, width, height, 0, 0, 0);
+//win = DefaultRootWindow (dpy);
 {
 XWindowAttributes wa;
 XVisualInfo model;
 XVisualInfo *vis_info;
 int nvisuals;
-xpv_t mask;
 
 XGetWindowAttributes (dpy, win, &wa);
 win_vis   = wa.visual;
@@ -315,6 +344,7 @@ verbmsg (" colours %d  masks %08lX %08lX %08lX\n",
    and >> sign extends. */
 if (win_r_mask)
    {
+   xpv_t mask;
    for (mask = win_r_mask, win_r_ofs = 0; ! (mask & 1); mask >>= 1)
       win_r_ofs++;
    for (win_r_bits = 0; mask & 1; mask >>= 1)
@@ -322,6 +352,7 @@ if (win_r_mask)
    }
 if (win_g_mask)
    {
+   xpv_t mask;
    for (mask = win_g_mask, win_g_ofs = 0; ! (mask & 1); mask >>= 1)
       win_g_ofs++;
    for (win_g_bits = 0; mask & 1; mask >>= 1)
@@ -329,6 +360,7 @@ if (win_g_mask)
    }
 if (win_b_mask)
    {
+   xpv_t mask;
    for (mask = win_b_mask, win_b_ofs = 0; ! (mask & 1); mask >>= 1)
       win_b_ofs++;
    for (win_b_bits = 0; mask & 1; mask >>= 1)
@@ -339,6 +371,76 @@ verbmsg ("g_ofs %d  g_bits %d  ", win_g_ofs, win_g_bits);
 verbmsg ("b_ofs %d  b_bits %d\n", win_b_ofs, win_b_bits);
 }
 
+/*
+ *	Get info relevant to XImages
+ */
+ximage_bpp     = 0;
+ximage_quantum = 0;
+{
+int nformats = 0;
+XPixmapFormatValues *format = XListPixmapFormats (dpy, &nformats);
+if (format == 0 || nformats < 1)
+   {
+   warn ("XListPixmapFormats() trouble (ret=%p, n=%d).\n", format, nformats);
+   goto ximage_done;
+   }
+/* Pick the best possible pixmap format. It must have the same
+   depth as the drawable. Should there be several formats that
+   satisfy that requirement (I don't think that ever happens),
+   prefer the one where the line quantum is equal to the number
+   of bits per pixel, as that is easier to work with. */
+{
+const XPixmapFormatValues *best = 0;
+for (int n = 0; n < nformats; n++)
+   {
+   if (format[n].depth != win_depth)
+      continue;
+   best = format + n;
+   if (format[n].scanline_pad == format[n].bits_per_pixel)
+      break;
+   }
+if (best != 0)
+   {
+   int bits_per_pixel = best->bits_per_pixel;
+   if (bits_per_pixel % 8)  // Paranoia
+      {
+      round_up (bits_per_pixel, 8);
+      warn ("XImage format has bad bits_per_pixel %d. Rounding up to %d.\n",
+	 best->bits_per_pixel, bits_per_pixel);
+      }
+   int scanline_pad = best->scanline_pad;
+   if (best->scanline_pad % 8)  // Paranoia
+      {
+      round_up (scanline_pad, 8);
+      warn ("XImage format has bad scanline_pad %d. Rounding up to %d.\n",
+	 best->scanline_pad, scanline_pad);
+      }
+   ximage_bpp     = bits_per_pixel / 8;
+   ximage_quantum = scanline_pad   / 8;
+   }
+}
+if (ximage_bpp == 0 || ximage_quantum == 0)
+   {
+   warn ("XListPixmapFormats() returned not suitable formats.\n"); 
+   goto ximage_done;
+   }
+ximage_done:
+if (format != 0)
+   XFree (format);
+}
+/* Could not obtain authoritative/good values. Warn and guess
+   plausible values. */
+if (ximage_bpp == 0 || ximage_quantum == 0)
+   {
+   if (ximage_bpp == 0)
+      ximage_bpp = win_bpp;
+   if (ximage_quantum == 0)
+      ximage_quantum = 4;
+   warn ("guessing format. Images will probably not be displayed correctly\n");
+   }
+verbmsg ("XImage format: %d B per pixel, %d B quantum.\n",
+   ximage_bpp, ximage_quantum);
+
 /*
  *	Further configure the window
  */
@@ -366,23 +468,32 @@ XSelectInput (dpy, win,
 XFontStruct *xqf;
 
 // Load the font or use the default font.
+default_font = true;
 if (font_name != NULL)
    {
+   x_catch_on ();  // Catch errors in XLoadFont()
    font_xfont = XLoadFont (dpy, font_name);
-   XFlush (dpy);
+   if (const char *err_msg = x_error ())
+      {
+      warn ("can't load font \"%s\" (%s).\n", font_name, err_msg);
+      warn ("using default font instead.\n");
+      }
+   else
+      default_font = false;
+   x_catch_off ();
    }
 
 // Query the font we'll use for FONTW, FONTH and FONTYOFS.
 xqf = XQueryFont (dpy,
-   font_name ? font_xfont : XGContextFromGC (DefaultGC (dpy, scn)));
+   default_font ? XGContextFromGC (DefaultGC (dpy, scn)) : font_xfont);
 if (xqf->direction != FontLeftToRight)
-   printf ("Warning: this font is not left-to-right !\n");
+   warn ("this font is not left-to-right !\n");
 if (xqf->min_byte1 != 0 || xqf->max_byte1 != 0)
-   printf ("Warning: this is not a single-byte font !\n");
+   warn ("this is not a single-byte font !\n");
 if (xqf->min_char_or_byte2 > 32 || xqf->max_char_or_byte2 < 126)
-   printf ("Warning: this font does not support the ASCII character set !\n");
+   warn ("this font does not support the ASCII character set !\n");
 if (xqf->min_bounds.width != xqf->max_bounds.width)
-   printf ("Warning: this is not a fixed-width font !\n");
+   warn ("this is not a fixed-width font !\n");
 FONTW     = xqf->max_bounds.width;
 FONTH     = xqf->ascent + xqf->descent;
 font_xofs = xqf->min_bounds.lbearing;
@@ -407,49 +518,18 @@ else
 XSetWindowColormap (dpy, win, cmap);
 game_colour = alloc_game_colours (0);
 app_colour = commit_app_colours ();
+if (win_depth == 24)
+   game_colour_24.refresh (game_colour, x_server_big_endian);
 
 /*
  *	Create the GC
  */
-#ifdef MANY_GC
-for (size_t acn = 0; acn < NCOLOURS; acn++)
-   {
-   XGCValues gcv;
-   unsigned long mask;
-
-   mask = GCForeground | GCFunction | GCLineWidth;
-   if (font_name != NULL)
-      {
-      mask |= GCFont;
-      gcv.font = font_xfont;
-      }
-
-   gcv.foreground = app_colour[acn];
-
-   // For each colour, create 4 GC
-   gcv.line_width    = 0;
-   gcv.function      = GXcopy;
-   std_gc[0][0][acn] = XCreateGC (dpy, win, mask, &gcv);
-
-   gcv.line_width    = 1;  // Important! See note 1
-   gcv.function      = GXxor;
-   std_gc[0][1][acn] = XCreateGC (dpy, win, mask, &gcv);
-
-   gcv.line_width    = 3;
-   gcv.function      = GXcopy;
-   std_gc[1][0][acn] = XCreateGC (dpy, win, mask, &gcv);
-
-   gcv.function      = GXxor;
-   std_gc[1][1][acn] = XCreateGC (dpy, win, mask, &gcv);
-   }
-gc = std_gc[0][0][0];
-#else
 {
 XGCValues gcv;
 unsigned long mask;
 
 mask = GCForeground | GCFunction | GCLineWidth;
-if (font_name != NULL)
+if (! default_font)
    {
    mask |= GCFont;
    gcv.font = font_xfont;
@@ -458,8 +538,9 @@ gcv.foreground = app_colour[0];  // Default colour
 gcv.line_width = 0;
 gcv.function   = GXcopy;
 gc = XCreateGC (dpy, win, mask, &gcv);
+if (gc == 0)
+   fatal_error ("XCreateGC() returned NULL");
 }
-#endif
 
 
 /*
@@ -483,11 +564,12 @@ else
    drw = win;
    drw_mods = 0;  // Force display the first time
    }
-//XSync (dpy, False);
+XSync (dpy, False);
 GfxMode = - VideoMode;
 #endif
 
 SetWindowSize (width, height);
+return 0;
 }
 
 
@@ -522,23 +604,13 @@ if (GfxMode)
       verbmsg ("  Freeing Colormap\n");
       XFreeColormap  (dpy, cmap);
       }
-   if (font_name != NULL)
+   if (! default_font)
       {
       verbmsg ("  Unloading font\n");
       XUnloadFont (dpy, font_xfont);
       }
-#ifdef MANY_GC
-   for (int n = 0; n < NCOLOURS; n++)
-      {
-      XFreeGC (dpy, std_gc[0][0][n]);
-      XFreeGC (dpy, std_gc[0][1][n]);
-      XFreeGC (dpy, std_gc[1][0][n]);
-      XFreeGC (dpy, std_gc[1][1][n]);
-      }
-#else
    XFreeGC (dpy, gc);
    gc = 0;
-#endif
    /* FIXME there is surely more to do ... */
    XCloseDisplay (dpy);
 #endif
@@ -721,6 +793,18 @@ drw = win;  // If they don't like it, they can call ClearScreen() [HHOS]
 }
 
 
+/*
+ *	force_window_not_pixmap
+ *	Redirect graphic output to window, not pixmap.
+ *	Used only in yadex.cc, before calling the sprite viewer.
+ *	FIXME this is not a clean way to do things.
+ */
+void force_window_not_pixmap ()
+{
+drw = win;
+}
+
+
 /*
  *	set_pcolour
  *	Set the current drawing colour
@@ -734,6 +818,9 @@ XSetForeground (dpy, gc, (xpv_t) colour);
 
 // Last application colour set by set_colour()
 static acolour_t current_acolour = 0;
+#ifdef Y_BGI
+static int current_bgicolour = 0;
+#endif
 
 
 /*
@@ -744,19 +831,17 @@ static acolour_t current_acolour = 0;
 void set_colour (acolour_t colour)
 {
 #if defined Y_BGI
-if (GfxMode < 0)
-   setcolor (TranslateToDoomColor (colour));
-else
-   setcolor (colour);
+if (colour != current_acolour)
+   {
+   current_acolour = colour;
+   current_bgicolour = (GfxMode < 0) ? TranslateToDoomColor (colour) : colour;
+   setcolor (current_bgicolour);
+   }
 #elif defined Y_X11
 if (colour != current_acolour)
    {
    current_acolour = colour;
-#ifdef MANY_GC
-   // gc = std_gc[LineThickness][DrawingMode][colour];
-#else
    XSetForeground (dpy, gc, app_colour[colour]);
-#endif
    }
 #endif
 }
@@ -769,8 +854,11 @@ if (colour != current_acolour)
  */
 void push_colour (acolour_t colour)
 {
-if (colour_stack_pointer >= sizeof colour_stack / sizeof *colour_stack)
-   fatal_error ("Colour stack overflow");
+if (colour_stack_pointer >= (int) (sizeof colour_stack / sizeof *colour_stack))
+   {
+   nf_bug ("Colour stack overflow");
+   return;
+   }
 colour_stack[colour_stack_pointer] = current_acolour;
 colour_stack_pointer++;
 set_colour (colour);
@@ -784,7 +872,10 @@ set_colour (colour);
 void pop_colour (void)
 {
 if (colour_stack_pointer < 1)
-   fatal_error ("Colour stack underflow");
+   {
+   nf_bug ("Colour stack underflow");
+   return;
+   }
 colour_stack_pointer--;
 set_colour (colour_stack[colour_stack_pointer]);
 }
@@ -802,17 +893,13 @@ setlinestyle (SOLID_LINE, 0, thick ? THICK_WIDTH : NORM_WIDTH);
 if (!! thick != LineThickness)
    {
    LineThickness = !! thick;
-#ifdef MANY_GC
-   gc = std_gc[LineThickness][DrawingMode][current_acolour];
-#else
    XGCValues gcv;
    gcv.line_width = LineThickness ? 3 : (DrawingMode ? 1 : 0);
    // ^ It's important to use a line_width of 1 when in xor mode.
-   // See note (1) in the developper's guide.
+   // See note (1) in the hacker's guide.
    XChangeGC (dpy, gc, GCLineWidth, &gcv);
    }
 #endif
-#endif
 }
 
 
@@ -820,37 +907,47 @@ if (!! thick != LineThickness)
    set the drawing mode (copy or xor)
 */
 
-void SetDrawingMode (int xor)
+void SetDrawingMode (int _xor)
 {
 #if defined Y_BGI
-setwritemode (xor ? XOR_PUT : COPY_PUT);
+setwritemode (_xor ? XOR_PUT : COPY_PUT);
 #elif defined Y_X11
-if (!! xor != DrawingMode)
+if (!! _xor != DrawingMode)
    {
-   DrawingMode = !! xor;
-#ifdef MANY_GC
-   gc = std_gc[LineThickness][DrawingMode][current_acolour];
-#else
+   DrawingMode = !! _xor;
    XGCValues gcv;
    gcv.function = DrawingMode ? GXxor : GXcopy;
    gcv.line_width = LineThickness ? 3 : (DrawingMode ? 1 : 0);
    // ^ It's important to use a line_width of 1 when in xor mode.
-   // See note (1) in the developper's guide.
+   // See note (1) in the hacker's guide.
    XChangeGC (dpy, gc, GCFunction | GCLineWidth, &gcv);
    }
 #endif
-#endif
 }
 
 
 /*
+ *	draw_point - draw a point at display coordinates (<x>, <y>)
+ */
+void draw_point (int x, int y)
+{
+#if defined Y_BGI
+putpixel (x, y, current_bgicolour);
+#elif defined Y_X11
+XDrawPoint (dpy, drw, gc, x, y);
+#endif
+}
+
+
+/*
+
  *	draw_map_point
  *	Draw a point at map coordinates <mapx>, <mapy>
  */
 void draw_map_point (int mapx, int mapy)
 {
 #if defined Y_BGI
-printf ("draw_map_point unimplemented\n");
+putpixel (SCREENX (mapx), SCREENY (mapy), current_bgicolour);
 #elif defined Y_X11
 XDrawPoint (dpy, drw, gc, SCREENX (mapx), SCREENY (mapy));
 drw_mods++;
@@ -904,21 +1001,20 @@ int    scry1   = SCREENY (mapy1);
 int    scrx2   = SCREENX (mapx2);
 int    scry2   = SCREENY (mapy2);
 double r       = hypot ((double) (scrx1 - scrx2), (double) (scry1 - scry2));
-/* AYM 19980216 to avoid getting huge arrow when zooming in */
+/* AYM 19980216 to avoid getting huge arrowheads when zooming in */
 int    scrXoff = (r >= 1.0) ? (int) ((scrx1 - scrx2) * 8.0 / r * (Scale < 1 ? Scale : 1)) : 0;
 int    scrYoff = (r >= 1.0) ? (int) ((scry1 - scry2) * 8.0 / r * (Scale < 1 ? Scale : 1)) : 0;
 
 #if defined Y_BGI
 line (scrx1, scry1, scrx2, scry2);
-#elif defined Y_X11
-XDrawLine (dpy, drw, gc, scrx1, scry1, scrx2, scry2);
-#endif
 scrx1 = scrx2 + 2 * scrXoff;
 scry1 = scry2 + 2 * scrYoff;
-#if defined Y_BGI
 line (scrx1 - scrYoff, scry1 + scrXoff, scrx2, scry2);
 line (scrx1 + scrYoff, scry1 - scrXoff, scrx2, scry2);
 #elif defined Y_X11
+XDrawLine (dpy, drw, gc, scrx1, scry1, scrx2, scry2);
+scrx1 = scrx2 + 2 * scrXoff;
+scry1 = scry2 + 2 * scrYoff;
 XDrawLine (dpy, drw, gc, scrx1 - scrYoff, scry1 + scrXoff, scrx2, scry2);
 XDrawLine (dpy, drw, gc, scrx1 + scrYoff, scry1 - scrXoff, scrx2, scry2);
 drw_mods++;
@@ -945,15 +1041,14 @@ int    scrYoff = (r >= 1.0) ? (int) ((scry1 - scry2) * 8.0 / r * (Scale < 1 ? Sc
 
 #if defined Y_BGI
 line (scrx1, scry1, scrx2, scry2);
-#elif defined Y_X11
-XDrawLine (dpy, drw, gc, scrx1, scry1, scrx2, scry2);
-#endif
 scrx1 = scrx2 + 2 * scrXoff;
 scry1 = scry2 + 2 * scrYoff;
-#if defined Y_BGI
 line (scrx1 - scrYoff, scry1 + scrXoff, scrx2, scry2);
 line (scrx1 + scrYoff, scry1 - scrXoff, scrx2, scry2);
 #elif defined Y_X11
+XDrawLine (dpy, drw, gc, scrx1, scry1, scrx2, scry2);
+scrx1 = scrx2 + 2 * scrXoff;
+scry1 = scry2 + 2 * scrYoff;
 XDrawLine (dpy, drw, gc, scrx1 - scrYoff, scry1 + scrXoff, scrx2, scry2);
 XDrawLine (dpy, drw, gc, scrx1 + scrYoff, scry1 - scrXoff, scrx2, scry2);
 drw_mods++;
@@ -977,18 +1072,58 @@ drw_mods++;
 }
 
 
+void DrawScreenLineLen (int x, int y, int width, int height)
+{
+#if defined Y_BGI
+line (x, y, x + width - 1, y + height - 1);
+#elif defined Y_X11
+if (width > 0)
+  width--;
+else if (width < 0)
+  width++;
+if (height > 0)
+  height--;
+else if (height < 0)
+  height++;
+XDrawLine (dpy, drw, gc, x, y, x + width, y + height);
+drw_mods++;
+#endif
+}
+
+
+/*
+ *	DrawScreenRect
+ *	Draw a rectangle
+ *	Unlike most functions here, the 3rd and 4th parameters
+ *	specify lengths, not coordinates.
+ */
+void DrawScreenRect (int x, int y, int width, int height)
+{
+#if defined Y_BGI
+rectangle (x, y, x + width - 1, y + height - 1);
+#elif defined Y_X11
+XDrawRectangle (dpy, drw, gc, x, y, width - 1, height - 1);
+drw_mods++;
+#endif
+}
+
 
 /*
    draw a filled in box on the screen from screen coords
+   (scrx1, scry1) is the top left corner
+   (scrx2, scry2) is the bottom right corner
+   If scrx2 < scrx1 or scry2 < scry1, the function does nothing.
 */
 
 void DrawScreenBox (int scrx1, int scry1, int scrx2, int scry2)
 {
+if (scrx2 < scrx1 || scry2 < scry1)
+  return;
 #if defined Y_BGI
 setfillstyle (1, getcolor ());
 bar (scrx1, scry1, scrx2, scry2);
 #elif defined Y_X11
-/* FIXME missing gc fill_style */
+// FIXME missing gc fill_style
 XFillRectangle (dpy, drw, gc, scrx1, scry1,
   scrx2 - scrx1 + 1, scry2 - scry1 + 1);
 drw_mods++;
@@ -996,6 +1131,27 @@ drw_mods++;
 }
 
 
+/*
+ *	DrawScreenBoxwh
+ *	Draw a filled rectangle of width x height pixels
+ *	(scrx0, scry0) is the top left corner
+ *	(width, height) is the obvious
+ *	If width < 1 or height < 1, does nothing.
+ */
+void DrawScreenBoxwh (int scrx0, int scry0, int width, int height)
+{
+if (width < 1 || height < 1)
+   return;
+#if defined Y_BGI
+setfillstyle (1, getcolor ());
+bar (scrx0, scry0, scrx0 + width - 1, scry0 + height - 1);
+#elif defined Y_X11
+// FIXME missing gc fill_style
+XFillRectangle (dpy, drw, gc, scrx0, scry0, width, height);
+drw_mods++;
+#endif
+}
+
 
 /*
    draw a filled-in 3D-box on the screen from screen coords
@@ -1071,6 +1227,7 @@ void draw_box_border (int x, int y, int width, int height,
    int thickness, int raised)
 {
 #if defined Y_BGI
+#error "You need to write the BGI version of draw_box_border()"
 #elif defined Y_X11
 int n;
 XPoint points[3];
@@ -1176,6 +1333,17 @@ printf ("DrawScreenMeter()\n");  /* FIXME ! */
 }
 
 
+// Shared by DrawScreenText() and DrawScreenString()
+
+/* Coordinates of first character of last string drawn with
+   DrawScreenText() or DrawScreenString(). */
+static int lastx0;
+static int lasty0;
+
+/* Where the last DrawScreen{Text,String}() left the "cursor". */
+static int lastxcur;
+static int lastycur;
+
 
 /*
  *	DrawScreenText
@@ -1188,59 +1356,82 @@ printf ("DrawScreenMeter()\n");  /* FIXME ! */
  *	If <msg> == NULL, no text is printed. Useful to set the
  *	coordinates for the next time.
  */
-
 void DrawScreenText (int scrx, int scry, const char *msg, ...)
 {
-static int lastX;
-static int lastY;
 char temp[120];
 va_list args;
 
 // <msg> == NULL: print nothing, just set the coordinates.
 if (msg == NULL)
    {
-   if (scrx != -1)
-      lastX = scrx;
-   if (scry != -1)
-      lastY = scry;  // Note: no "+ FONTH"
+   if (scrx != -1 && scrx != -2)
+      {
+      lastx0   = scrx;
+      lastxcur = scry;
+      }
+   if (scry != -1 && scry != -2)
+      {
+      lasty0   = scry;  // Note: no "+ FONTH"
+      lastycur = scry;
+      }
    return;
    }
 
 va_start (args, msg);
-#ifdef Y_SNPRINTF
-vsnprintf (temp, sizeof temp, msg, args);
-#else
-vsprintf (temp, msg, args);
-#endif
-va_end (args);
+y_vsnprintf (temp, sizeof temp, msg, args);
+DrawScreenString (scrx, scry, temp);
+}
+
+
+/*
+ *	DrawScreenString
+ *	Same thing as DrawScreenText() except that the string is
+ *	printed verbatim (no formatting or conversion).
+ */
+void DrawScreenString (int scrx, int scry, const char *str)
+{
+int x;
+int y;
 
 /* FIXME originally, the test was "< 0". Because it broke
    when the screen was too small, I changed it to a more
    specific "== -1". A quick and very dirty hack ! */
 if (scrx == -1)
-   scrx = lastX;
+   x = lastx0;
+else if (scrx == -2)
+   x = lastxcur;
+else
+   x = scrx;
 if (scry == -1)
-   scry = lastY;
+   y = lasty0;
+else if (scry == -2)
+   y = lastycur;
+else
+   y = scry;
+const size_t len = strlen (str);
 #if defined Y_BGI
-outtextxy (scrx, scry, temp);
+outtextxy (x, y, str);
 #elif defined Y_X11
-XDrawString (dpy, drw, gc, scrx - font_xofs, scry + font_yofs,
-   temp, strlen (temp));
+XDrawString (dpy, drw, gc, x - font_xofs, y + font_yofs, str, len);
 if (text_dot)
-   XDrawPoint (dpy, drw, gc, scrx, scry);
+   XDrawPoint (dpy, drw, gc, x, y);
 drw_mods++;
 #endif
-lastX = scrx;
-lastY = scry + FONTH;
-}
 
+lastxcur = x + FONTW * len;
+lastycur = y;
+if (scrx != -2)
+   lastx0 = x;
+if (scry != -2)
+   lasty0 = y + FONTH;
+}
 
 
 /*
    draw (or erase) the pointer if we aren't using the mouse
 */
 
-void DrawPointer (Bool rulers)
+void DrawPointer (bool rulers)
 {
 #ifdef Y_BGI
 int r;
@@ -1427,8 +1618,8 @@ wrinx (SEQ, 0x13, 0x3F);
 
 
 #ifdef Y_DOS
-int cooked_installuserdriver (const char far *__name,
-                             int huge (*detect)(void))
+static int cooked_installuserdriver (const char far *__name,
+                                    int huge (*detect)(void))
 {
 char savecwd[PATH_MAX+1];
 int gdriver;
