@@ -7,25 +7,23 @@
 /*
 This file is part of Yadex.
 
-Yadex incorporates code from DEU 5.21 that was put in the public
-domain in 1994 by Raphal Quinet and Brendon Wyber.
+Yadex incorporates code from DEU 5.21 that was put in the public domain in
+1994 by Raphal Quinet and Brendon Wyber.
 
-The rest of Yadex is Copyright  1997-1999 Andr Majorel.
+The rest of Yadex is Copyright  1997-2000 Andr Majorel.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
-You should have received a copy of the GNU Library General Public
-License along with this library; if not, write to the Free
-Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307, USA.
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place, Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
@@ -219,7 +217,7 @@ while ((num < 36) && ((menustr[num] = va_arg (args, char *)) != NULL))
    menu_flags_t flags;
    shortcut_index[num] = va_arg (args, int);
    shortcut_key  [num] = va_arg (args, int);
-   flags = va_arg (args, menu_flags_t);
+   flags = (menu_flags_t) va_arg (args, int);
    if (flags & MEN_TICK)
       {
       has_tick = 1;
@@ -323,8 +321,8 @@ if (delete_after_use)
  *	set_coords
  *	Position or reposition the menu window.
  *	(<x0>,<y0>) is the top left corner.
- *	If <x0> is < 0, the window is horizontally centered.
- *	If <y0> is < 0, the window is vertically centered.
+ *	If <x0> is < 0, the window is horizontally centred.
+ *	If <y0> is < 0, the window is vertically centred.
  */
 void menu_c::set_coords (int x0, int y0)
 {
@@ -372,13 +370,13 @@ if (this->grayed_out)
  */
 void menu_c::geom ()
 {
-int width_chars = 0;
+size_t width_chars = 0;
 if (title && popup)
-   width_chars = max (width_chars, title_len);
+   width_chars = y_max (width_chars, title_len);
 if (force_numbers)
-   width_chars = max (width_chars, items_len + 4);
+   width_chars = y_max (width_chars, items_len + 4);
 else
-   width_chars = max (width_chars, items_ks_len);
+   width_chars = y_max (width_chars, items_ks_len);
 int title_height = title && popup ? (int) (1.5 * FONTH) : 0;
 
 width  = 2 * BOX_BORDER + 2 * WIDE_HSPACING + width_chars * FONTW;
@@ -668,7 +666,7 @@ if (from_scratch)
    DrawScreenBox3D (ox0, oy0, ox1, oy1);
    set_colour (YELLOW);
    if (popup && title != NULL)
-      DrawScreenText (ix0 + WIDE_HSPACING, ty0, title);
+      DrawScreenString (ix0 + WIDE_HSPACING, ty0, title);
    ShowMousePointer ();
 
    for (use_list ?   al_lrewind (list) : 0,            l = 0;
@@ -743,22 +741,22 @@ DrawScreenBox (ix0, y, ix1, y + FONTH - 1);
 set_colour (menu_colour[grayed_out][highlighted].fg);
 if (ticked)
    {
-   DrawScreenText (x, y, ticked[line] ? "*" : " ");
+   DrawScreenString (x, y, ticked[line] ? "*" : " ");
    x += 2 * FONTW;
    }
 if (force_numbers || ! this->shortcut_index)
    {
    DrawScreenText (x, y, "(%c)", al_adigits[line + 1]);
-   DrawScreenText (x + FONTW, y + FONTU, "_");
+   DrawScreenString (x + FONTW, y + FONTU, "_");
    x += 4 * FONTW;
    }
-DrawScreenText (x, y, "%s", text);
+DrawScreenString (x, y, text);
 if (! force_numbers && shortcut_index >= 0)
-   DrawScreenText (x + shortcut_index * FONTW, y + FONTU, "_");
+   DrawScreenString (x + shortcut_index * FONTW, y + FONTU, "_");
 if (! force_numbers && shortcut_key != -1)
    {
    const char *string = key_to_string (shortcut_key);
-   DrawScreenText (ix1 + 1 - FONTW - strlen (string) * FONTW, y, string);
+   DrawScreenString (ix1 + 1 - FONTW - strlen (string) * FONTW, y, string);
    }
 ShowMousePointer ();
 }
