@@ -8,25 +8,23 @@
 /*
 This file is part of Yadex.
 
-Yadex incorporates code from DEU 5.21 that was put in the public
-domain in 1994 by Raphal Quinet and Brendon Wyber.
+Yadex incorporates code from DEU 5.21 that was put in the public domain in
+1994 by Raphal Quinet and Brendon Wyber.
 
-The rest of Yadex is Copyright  1997-1999 Andr Majorel.
+The rest of Yadex is Copyright  1997-2000 Andr Majorel.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
-You should have received a copy of the GNU Library General Public
-License along with this library; if not, write to the Free
-Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307, USA.
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place, Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
@@ -59,7 +57,7 @@ for (cur = list; cur; cur = cur->next)
 /*
    get the number of objets of a given type minus one
 */
-int GetMaxObjectNum (int objtype)
+obj_no_t GetMaxObjectNum (int objtype)
 {
 switch (objtype)
    {
@@ -258,8 +256,13 @@ switch (objtype)
 	 methods fails miserably. I suppose that the solution would be to look
 	 for intersections in BOTH directions and pick the closest. Remind me
 	 to look into it one of these days :-). */
+      /* AYM 1999-03-18
+         Initializing curx to 32767 instead of MapMaxX + 1. Fixes the old
+	 DEU bug where sometimes you couldn't select a newly created sector
+	 to the west of the level until you saved. (MapMaxX got out of date
+	 and SaveLevelData() refreshed it.) */
       ObjectsNeeded (OBJ_LINEDEFS, OBJ_VERTICES, 0);
-      curx = MapMaxX + 1;
+      curx = 32767;  // Oh yes, one more hard-coded constant!
       for (n = 0; n < NumLineDefs; n++)
 	 if ((Vertices[LineDefs[n].start].y > y)
 	  != (Vertices[LineDefs[n].end].y > y))
@@ -276,7 +279,7 @@ switch (objtype)
 	       best_match = n;
 	       }
 	    }
-      /* now look if this LineDef has a SideDef bound to one sector */
+      /* now look if this linedef has a sidedef bound to one sector */
       if (best_match >= 0)
 	 {
 	 if (Vertices[LineDefs[best_match].start].y
@@ -302,116 +305,6 @@ return best_match;
 
 
 
-/*
-   select all objects inside a given box
-*/
-
-void SelectObjectsInBox (SelPtr *list, int objtype, int x0, int y0, int x1, int y1) /* SWAP! */
-{
-int n, m;
-
-if (x1 < x0)
-   {
-   n = x0;
-   x0 = x1;
-   x1 = n;
-   }
-if (y1 < y0)
-   {
-   n = y0;
-   y0 = y1;
-   y1 = n;
-   }
-
-switch (objtype)
-   {
-   case OBJ_THINGS:
-      ObjectsNeeded (OBJ_THINGS, 0);
-      for (n = 0; n < NumThings; n++)
-	 if (Things[n].xpos >= x0 && Things[n].xpos <= x1
-	  && Things[n].ypos >= y0 && Things[n].ypos <= y1)
-            select_unselect_obj (list, n);
-      break;
-   case OBJ_VERTICES:
-      ObjectsNeeded (OBJ_VERTICES, 0);
-      for (n = 0; n < NumVertices; n++)
-	 if (Vertices[n].x >= x0 && Vertices[n].x <= x1
-	  && Vertices[n].y >= y0 && Vertices[n].y <= y1)
-            select_unselect_obj (list, n);
-      break;
-   case OBJ_LINEDEFS:
-      ObjectsNeeded (OBJ_LINEDEFS, OBJ_VERTICES, 0);
-      for (n = 0; n < NumLineDefs; n++)
-	 {
-	 /* the two ends of the line must be in the box */
-	 m = LineDefs[n].start;
-	 if (Vertices[m].x < x0 || Vertices[m].x > x1
-	  || Vertices[m].y < y0 || Vertices[m].y > y1)
-	    continue;
-	 m = LineDefs[n].end;
-	 if (Vertices[m].x < x0 || Vertices[m].x > x1
-	  || Vertices[m].y < y0 || Vertices[m].y > y1)
-	    continue;
-         select_unselect_obj (list, n);
-	 }
-      break;
-   case OBJ_SECTORS:
-      {
-      signed char *sector_status;
-      LDPtr ld;
-
-      sector_status = (signed char *) GetMemory (NumSectors);
-      memset (sector_status, 0, NumSectors);
-      for (n = 0, ld = LineDefs; n < NumLineDefs; n++, ld++)
-         {
-         VPtr v1, v2;
-         int s1, s2;
-
-         v1 = Vertices + ld->start;
-         v2 = Vertices + ld->end;
-
-         // Get the numbers of the sectors on both sides of the linedef
-         s1 = is_sector (SideDefs[ld->sidedef1].sector)
-            ? SideDefs[ld->sidedef1].sector : OBJ_NO_NONE;
-         s2 = is_sector (SideDefs[ld->sidedef2].sector)
-            ? SideDefs[ld->sidedef2].sector : OBJ_NO_NONE;
-
-         // The linedef is entirely within bounds.
-         // The sectors it touches _might_ be within bounds too.
-         if (v1->x >= x0 && v1->x <= x1
-          && v1->y >= y0 && v1->y <= y1
-          && v2->x >= x0 && v2->x <= x1
-          && v2->y >= y0 && v2->y <= y1)
-            {
-            if (is_sector (s1) && sector_status[s1] >= 0)
-               sector_status[s1] = 1;
-            if (is_sector (s2) && sector_status[s2] >= 0)
-               sector_status[s2] = 1;
-            }
-
-         // It's not. The sectors it touches _can't_ be within bounds.
-         else
-            {
-            if (is_sector (s1))
-               sector_status[s1] = -1;
-            if (is_sector (s2))
-               sector_status[s2] = -1;
-            }
-         }
-
-      // Now select all the sectors we found to be within bounds
-      ForgetSelection (list);
-      for (n = 0; n < NumSectors; n++)
-         if (sector_status[n] > 0)
-            SelectObject (list, n);
-      FreeMemory (sector_status);
-      }
-      break;
-   }
-}
-
-
-
 /*
    highlight the selected object
 */
@@ -465,7 +358,7 @@ switch (objtype)
    case OBJ_VERTICES:
       ObjectsNeeded (OBJ_VERTICES, 0);
       {
-      int r = (int) (2 * OBJSIZE * min (Scale, 1));
+      int r = (int) (2 * OBJSIZE * y_min (Scale, 1));
       int scrx0 = SCREENX (Vertices[objnum].x) - r;
       int scrx9 = SCREENX (Vertices[objnum].x) + r;
       int scry0 = SCREENY (Vertices[objnum].y) - r;
@@ -532,10 +425,20 @@ switch (objtype)
    {
    case OBJ_THINGS:
       ObjectsNeeded (OBJ_THINGS, 0);
+      if (*list)
+	 {
+	 things_angles++;
+	 things_types++;
+	 }
       while (*list)
 	 {
 	 objnum = (*list)->objnum;
-	 /* delete the Thing */
+	 if (objnum < 0 || objnum >= NumThings)  // Paranoia
+	 {
+	    nf_bug ("attempt to delete non-existent thing #%d", objnum);
+	    goto next_thing;
+	 }
+	 // Delete the thing
 	 NumThings--;
 	 if (NumThings > 0)
 	    {
@@ -552,15 +455,22 @@ switch (objtype)
 	 for (cur = (*list)->next; cur; cur = cur->next)
 	    if (cur->objnum > objnum)
 	       cur->objnum--;
+	 next_thing:
 	 UnSelectObject (list, objnum);
 	 }
       break;
+
    case OBJ_VERTICES:
       if (*list)
          MadeMapChanges = 1;
       while (*list)
 	 {
 	 objnum = (*list)->objnum;
+	 if (objnum < 0 || objnum >= NumVertices)  // Paranoia
+	 {
+	    nf_bug ("attempt to delete non-existent vertex #%d", objnum);
+	    goto next_vertex;
+	 }
 	 // Delete the linedefs bound to this vertex and change the references
 	 ObjectsNeeded (OBJ_LINEDEFS, 0);
 	 for (n = 0; n < NumLineDefs; n++)
@@ -575,7 +485,7 @@ switch (objtype)
 		  LineDefs[n].end--;
 	       }
 	    }
-	 /* delete the vertex */
+	 // Delete the vertex
 	 ObjectsNeeded (OBJ_VERTICES, 0);
 	 NumVertices--;
 	 if (NumVertices > 0)
@@ -593,9 +503,11 @@ switch (objtype)
 	 for (cur = (*list)->next; cur; cur = cur->next)
 	    if (cur->objnum > objnum)
 	       cur->objnum--;
+	 next_vertex:
 	 UnSelectObject (list, objnum);
 	 }
       break;
+
    case OBJ_LINEDEFS:
       /* In DEU, deleting a linedef was not considered to be a
       map change. Deleting a _sidedef_ was. In Yadex, sidedefs
@@ -606,14 +518,18 @@ switch (objtype)
          MadeMapChanges = 1;
       /* AYM 19980203 I've removed the deletion of sidedefs
          because if several linedefs use the same sidedef, this
-         lead would to trouble. Instead, I let the xref checking
+         would lead to trouble. Instead, I let the xref checking
          take care of that. */
       while (*list)
 	 {
 	 ObjectsNeeded (OBJ_LINEDEFS, 0);
 	 objnum = (*list)->objnum;
-
-	 /* delete the LineDef */
+	 if (objnum < 0 || objnum >= NumLineDefs)  // Paranoia
+	 {
+	    nf_bug ("attempt to delete non-existent linedef #%d", objnum);
+	    goto next_linedef;
+	 }
+	 // delete the linedef
 	 NumLineDefs--;
 	 if (NumLineDefs > 0)
 	    {
@@ -630,16 +546,23 @@ switch (objtype)
 	 for (cur = (*list)->next; cur; cur = cur->next)
 	    if (cur->objnum > objnum)
 	       cur->objnum--;
+	 next_linedef:
 	 UnSelectObject (list, objnum);
 	 }
       break;
+
    case OBJ_SIDEDEFS:
       if (*list)
          MadeMapChanges = 1;
       while (*list)
 	 {
 	 objnum = (*list)->objnum;
-	 /* change the LineDefs references */
+	 if (objnum < 0 || objnum >= NumSideDefs)  // Paranoia
+	 {
+	    nf_bug ("attempt to delete non-existent sidedef #%d", objnum);
+	    goto next_sidedef;
+	 }
+	 /* change the linedefs references */
 	 ObjectsNeeded (OBJ_LINEDEFS, 0);
 	 for (n = 0; n < NumLineDefs; n++)
 	    {
@@ -652,7 +575,7 @@ switch (objtype)
 	    else if (LineDefs[n].sidedef2 >= objnum)
 	       LineDefs[n].sidedef2--;
 	    }
-	 /* delete the SideDef */
+	 /* delete the sidedef */
 	 ObjectsNeeded (OBJ_SIDEDEFS, 0);
 	 NumSideDefs--;
 	 if (NumSideDefs > 0)
@@ -670,6 +593,7 @@ switch (objtype)
 	 for (cur = (*list)->next; cur; cur = cur->next)
 	    if (cur->objnum > objnum)
 	       cur->objnum--;
+	 next_sidedef:
 	 UnSelectObject (list, objnum);
 	 }
       break;
@@ -677,6 +601,11 @@ switch (objtype)
       while (*list)
 	{
 	objnum = (*list)->objnum;
+	 if (objnum < 0 || objnum >= NumSectors)  // Paranoia
+	 {
+	    nf_bug ("attempt to delete non-existent sector #%d", objnum);
+	    goto next_sector;
+	 }
 	// Delete the sidedefs bound to this sector and change the references
 	// AYM 19980203: Hmm, hope this is OK with multiply used sidedefs...
 	ObjectsNeeded (OBJ_SIDEDEFS, 0);
@@ -685,7 +614,7 @@ switch (objtype)
 	      DeleteObject (OBJ_SIDEDEFS, n--);
 	   else if (SideDefs[n].sector >= objnum)
 	      SideDefs[n].sector--;
-	/* delete the Sector */
+	/* delete the sector */
 	ObjectsNeeded (OBJ_SECTORS, 0);
 	NumSectors--;
 	if (NumSectors > 0)
@@ -703,11 +632,12 @@ switch (objtype)
 	for (cur = (*list)->next; cur; cur = cur->next)
 	   if (cur->objnum > objnum)
 	      cur->objnum--;
+	next_sector:
 	UnSelectObject (list, objnum);
 	}
       break;
    default:
-      Beep ();
+      nf_bug ("DeleteObjects: bad objtype %d", (int) objtype);
    }
 }
 
@@ -743,6 +673,8 @@ switch (objtype)
 	 Things = (TPtr) GetFarMemory (sizeof (struct Thing));
       Things[last].xpos = xpos;
       Things[last].ypos = ypos;
+      things_angles++;
+      things_types++;
       if (is_obj (copyfrom))
 	 {
 	 Things[last].type  = Things[copyfrom].type;
@@ -756,6 +688,7 @@ switch (objtype)
 	 Things[last].when  = 0x07;
 	 }
       break;
+
    case OBJ_VERTICES:
       last = NumVertices++;
       if (last > 0)
@@ -775,6 +708,7 @@ switch (objtype)
 	 MapMaxY = Vertices[last].y;
       MadeMapChanges = 1;
       break;
+
    case OBJ_LINEDEFS:
       last = NumLineDefs++;
       if (last > 0)
@@ -801,6 +735,7 @@ switch (objtype)
       LineDefs[last].sidedef1 = OBJ_NO_NONE;
       LineDefs[last].sidedef2 = OBJ_NO_NONE;
       break;
+
    case OBJ_SIDEDEFS:
       last = NumSideDefs++;
       if (last > 0)
@@ -812,9 +747,9 @@ switch (objtype)
 	 {
 	 SideDefs[last].xoff = SideDefs[copyfrom].xoff;
 	 SideDefs[last].yoff = SideDefs[copyfrom].yoff;
-	 strncpy (SideDefs[last].tex1, SideDefs[copyfrom].tex1, 8);
-	 strncpy (SideDefs[last].tex2, SideDefs[copyfrom].tex2, 8);
-	 strncpy (SideDefs[last].tex3, SideDefs[copyfrom].tex3, 8);
+	 strncpy (SideDefs[last].tex1, SideDefs[copyfrom].tex1, WAD_TEX_NAME);
+	 strncpy (SideDefs[last].tex2, SideDefs[copyfrom].tex2, WAD_TEX_NAME);
+	 strncpy (SideDefs[last].tex3, SideDefs[copyfrom].tex3, WAD_TEX_NAME);
 	 SideDefs[last].sector = SideDefs[copyfrom].sector;
 	 }
       else
@@ -828,6 +763,7 @@ switch (objtype)
 	 }
       MadeMapChanges = 1;
       break;
+
    case OBJ_SECTORS:
       last = NumSectors++;
       if (last > 0)
@@ -839,8 +775,8 @@ switch (objtype)
 	 {
 	 Sectors[last].floorh  = Sectors[copyfrom].floorh;
 	 Sectors[last].ceilh   = Sectors[copyfrom].ceilh;
-	 strncpy (Sectors[last].floort, Sectors[copyfrom].floort, 8);
-	 strncpy (Sectors[last].ceilt, Sectors[copyfrom].ceilt, 8);
+	 strncpy (Sectors[last].floort, Sectors[copyfrom].floort, WAD_FLAT_NAME);
+	 strncpy (Sectors[last].ceilt, Sectors[copyfrom].ceilt, WAD_FLAT_NAME);
 	 Sectors[last].light   = Sectors[copyfrom].light;
 	 Sectors[last].special = Sectors[copyfrom].special;
 	 Sectors[last].tag     = Sectors[copyfrom].tag;
@@ -849,15 +785,16 @@ switch (objtype)
 	 {
 	 Sectors[last].floorh  = default_floor_height;
 	 Sectors[last].ceilh   = default_ceiling_height;
-	 strncpy (Sectors[last].floort, default_floor_texture, 8);
-	 strncpy (Sectors[last].ceilt, default_ceiling_texture, 8);
+	 strncpy (Sectors[last].floort, default_floor_texture, WAD_FLAT_NAME);
+	 strncpy (Sectors[last].ceilt, default_ceiling_texture, WAD_FLAT_NAME);
 	 Sectors[last].light   = default_light_level;
 	 Sectors[last].special = 0;
 	 Sectors[last].tag     = 0;
 	 }
       break;
+
    default:
-      Beep ();
+      nf_bug ("InsertObject: bad objtype %d", (int) objtype);
    }
 }
 
@@ -867,7 +804,7 @@ switch (objtype)
    check if a (part of a) LineDef is inside a given block
 */
 
-Bool IsLineDefInside (int ldnum, int x0, int y0, int x1, int y1) /* SWAP - needs Vertices & LineDefs */
+bool IsLineDefInside (int ldnum, int x0, int y0, int x1, int y1) /* SWAP - needs Vertices & LineDefs */
 {
 int lx0 = Vertices[LineDefs[ldnum].start].x;
 int ly0 = Vertices[LineDefs[ldnum].start].y;
@@ -877,44 +814,44 @@ int i;
 
 /* do you like mathematics? */
 if (lx0 >= x0 && lx0 <= x1 && ly0 >= y0 && ly0 <= y1)
-   return 1; /* the LineDef start is entirely inside the square */
+   return 1; /* the linedef start is entirely inside the square */
 if (lx1 >= x0 && lx1 <= x1 && ly1 >= y0 && ly1 <= y1)
-   return 1; /* the LineDef end is entirely inside the square */
+   return 1; /* the linedef end is entirely inside the square */
 if ((ly0 > y0) != (ly1 > y0))
    {
    i = lx0 + (int) ((long) (y0 - ly0) * (long) (lx1 - lx0) / (long) (ly1 - ly0));
    if (i >= x0 && i <= x1)
-      return 1; /* the LineDef crosses the y0 side (left) */
+      return true; /* the linedef crosses the y0 side (left) */
    }
 if ((ly0 > y1) != (ly1 > y1))
    {
    i = lx0 + (int) ((long) (y1 - ly0) * (long) (lx1 - lx0) / (long) (ly1 - ly0));
    if (i >= x0 && i <= x1)
-      return 1; /* the LineDef crosses the y1 side (right) */
+      return true; /* the linedef crosses the y1 side (right) */
    }
 if ((lx0 > x0) != (lx1 > x0))
    {
    i = ly0 + (int) ((long) (x0 - lx0) * (long) (ly1 - ly0) / (long) (lx1 - lx0));
    if (i >= y0 && i <= y1)
-      return 1; /* the LineDef crosses the x0 side (down) */
+      return true; /* the linedef crosses the x0 side (down) */
    }
 if ((lx0 > x1) != (lx1 > x1))
    {
    i = ly0 + (int) ((long) (x1 - lx0) * (long) (ly1 - ly0) / (long) (lx1 - lx0));
    if (i >= y0 && i <= y1)
-      return 1; /* the LineDef crosses the x1 side (up) */
+      return true; /* the linedef crosses the x1 side (up) */
    }
-return 0;
+return false;
 }
 
 
 
 /*
-   get the Sector number of the SideDef opposite to this SideDef
+   get the sector number of the sidedef opposite to this sidedef
    (returns -1 if it cannot be found)
 */
 
-int GetOppositeSector (int ld1, Bool firstside) /* SWAP! */
+int GetOppositeSector (int ld1, bool firstside) /* SWAP! */
 {
 int x0, y0, dx0, dy0;
 int x1, y1, dx1, dy1;
@@ -1269,7 +1206,7 @@ switch (objtype)
  *
  *	Returns <>0 iff an object was moved.
  */
-Bool MoveObjectsToCoords (
+bool MoveObjectsToCoords (
    int objtype,
    SelPtr obj,
    int newx,
@@ -1292,7 +1229,7 @@ if (! obj)
    {
    refx = newx;
    refy = newy;
-   return 1;
+   return true;
    }
 
 /* compute the displacement */
@@ -1300,7 +1237,7 @@ dx = newx - refx;
 dy = newy - refy;
 /* nothing to do? */
 if (dx == 0 && dy == 0)
-   return 0;
+   return false;
 
 /* move the object(s) */
 switch (objtype)
@@ -1338,7 +1275,7 @@ switch (objtype)
       ForgetSelection (&vertices);
       break;
    }
-return 1;
+return true;
 }
 
 
@@ -1434,25 +1371,25 @@ switch (objtype)
 int FindFreeTag () /* SWAP! */
 {
 int  tag, n;
-Bool ok;
+bool ok;
 
 ObjectsNeeded (OBJ_LINEDEFS, OBJ_SECTORS, 0);
 tag = 1;
-ok = 0;
+ok = false;
 while (! ok)
    {
-   ok = 1;
+   ok = true;
    for (n = 0; n < NumLineDefs; n++)
       if (LineDefs[n].tag == tag)
 	 {
-	 ok = 0;
+	 ok = false;
 	 break;
 	 }
    if (ok)
       for (n = 0; n < NumSectors; n++)
 	 if (Sectors[n].tag == tag)
 	    {
-	    ok = 0;
+	    ok = false;
 	    break;
 	    }
    tag++;
