@@ -8,39 +8,51 @@
 /*
 This file is part of Yadex.
 
-Yadex incorporates code from DEU 5.21 that was put in the public
-domain in 1994 by Raphal Quinet and Brendon Wyber.
+Yadex incorporates code from DEU 5.21 that was put in the public domain in
+1994 by Raphal Quinet and Brendon Wyber.
 
-The rest of Yadex is Copyright  1997-1999 Andr Majorel.
+The rest of Yadex is Copyright  1997-2000 Andr Majorel.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free Software
+Foundation; either version 2 of the License, or (at your option) any later
+version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
-You should have received a copy of the GNU Library General Public
-License along with this library; if not, write to the Free
-Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307, USA.
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 59 Temple
+Place, Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
 #include "yadex.h"
 #include <time.h>
 #include "acolours.h"
+#include "bench.h"
 #include "cfgfile.h"
+#include "disppic.h"  /* Because of "p" */
 #include "editlev.h"
+#include "endian.h"
+#include "flats.h"
+#include "game.h"
 #include "gfx.h"
+#include "gfx2.h"
 #include "help1.h"
+#include "levels.h"    /* Because of "viewtex" */
+#include "lists.h"
 #include "mkpalette.h"
+#include "palview.h"
+#include "patchdir.h"  /* Because of "p" */
 #include "rgb.h"
 #include "sanity.h"
-#include "sprites.h"
+#include "textures.h"
+#include "x11.h"
+#include "wadname.h"
+#include "wadres.h"
+#include "wads2.h"
 
 
 /*
@@ -65,63 +77,82 @@ char error_invalid[1];     // Invalid parameter
  */
 const char *install_dir = 0;		// Where Yadex is installed
 FILE *      logfile     = NULL;		// Filepointer to the error log
-Bool        Registered  = 0;		// Registered or shareware game?
+bool        Registered  = false;	// Registered or shareware game?
 int         screen_lines = 24;  	// Lines that our TTY can display
 int         remind_to_build_nodes = 0;	// Remind user to build nodes
+Wad_res     wad_res (&MasterDir);
 
 // Set from command line and/or config file
-int       autoscroll                    = 1;
-int       autoscroll_amp                = 10;
-int       autoscroll_edge               = 30;
-char *    config_file;
-int       copy_linedef_reuse_sidedefs   = 0;
-Bool      Debug				= 0;
+int       autoscroll			= 0;
+int       autoscroll_amp		= 10;
+int       autoscroll_edge		= 30;
+const char *config_file;
+int       copy_linedef_reuse_sidedefs	= 0;
+int       cpu_big_endian		= 0;
+bool      Debug				= false;
 int       default_ceiling_height	= 128;
-char      default_ceiling_texture[9]	= "CEIL3_5";
-int       default_floor_height		= 0;
-char      default_floor_texture[9]	= "FLOOR4_8";
-int       default_light_level		= 144;
-char      default_lower_texture[9]	= "STARTAN3";
-char      default_middle_texture[9]	= "STARTAN3";
-char      default_upper_texture[9]	= "STARTAN3";
+char      default_ceiling_texture[WAD_FLAT_NAME + 1]	= "CEIL3_5";
+int       default_floor_height				= 0;
+char      default_floor_texture[WAD_FLAT_NAME + 1]	= "FLOOR4_8";
+int       default_light_level				= 144;
+char      default_lower_texture[WAD_TEX_NAME + 1]	= "STARTAN3";
+char      default_middle_texture[WAD_TEX_NAME + 1]	= "STARTAN3";
+char      default_upper_texture[WAD_TEX_NAME + 1]	= "STARTAN3";
 int       default_thing			= 3004;
-int       double_click_timeout            = 200;
-Bool      Expert			= 0;
-char *    Game				= NULL;
-int       grid_pixels_min               = 10;
-int       GridMin			= 4;
-int       GridMax			= 256;
-Bool      InfoShown			= 1;
-int       InitialScale			= 8;
+int       double_click_timeout		= 200;
+bool      Expert			= false;
+const char *Game			= NULL;
+int       grid_pixels_min		= 10;
+int       GridMin			= 2;
+int       GridMax			= 128;
+int       idle_sleep_ms			= 50;
+bool      InfoShown			= true;
+int       zoom_default			= 0;  // 0 means fit
+int       zoom_step			= 0;  // 0 means sqrt(2)
 confirm_t insert_vertex_split_linedef	= YC_ASK_ONCE;
 confirm_t insert_vertex_merge_vertices	= YC_ASK_ONCE;
-char *    Iwad1				= NULL;
-char *    Iwad2				= NULL;
-char *    Iwad3				= NULL;
-char *    MainWad			= NULL;
+const char *Iwad1			= NULL;
+const char *Iwad2			= NULL;
+const char *Iwad3			= NULL;
+const char *Iwad4			= NULL;
+const char *Iwad5			= NULL;
+const char *Iwad6			= NULL;
+const char *Iwad7			= NULL;
+const char *Iwad8			= NULL;
+const char *Iwad9			= NULL;
+const char *Iwad10			= NULL;
+const char *MainWad			= NULL;
 #ifdef AYM_MOUSE_HACKS
 int       MouseMickeysH			= 5;
 int       MouseMickeysV			= 5;
 #endif
 char **   PatchWads			= NULL;
-Bool      Quiet				= 0;
-Bool      Quieter			= 0;
+bool      Quiet				= false;
+bool      Quieter			= false;
 int       scroll_less			= 10;
 int       scroll_more			= 90;
-Bool      Select0			= 1;
+bool      Select0			= false;
 int       show_help			= 0;
-Bool      SwapButtons			= 0;
+int       sprite_scale                  = 100;
+bool      SwapButtons			= false;
 int       thing_fudge			= 400;
 int       verbose			= 0;
-int       vertex_fudge			= 400;
+int       vertex_fudge			= 200;
 int       welcome_message		= 1;
+const char *bench			= 0;
 
 // Global variables declared in game.h
-al_llist_t *ldtdef     = NULL;
-al_llist_t *ldtgroup   = NULL;
-al_llist_t *stdef      = NULL;
-al_llist_t *thingdef   = NULL;
-al_llist_t *thinggroup = NULL;
+yglf_t yg_level_format   = YGLF__;
+ygln_t yg_level_name     = YGLN__;
+ygpf_t yg_picture_format = YGPF_NORMAL;
+ygtf_t yg_texture_format = YGTF_NORMAL;
+ygtl_t yg_texture_lumps  = YGTL_NORMAL;
+al_llist_t *ldtdef       = NULL;
+al_llist_t *ldtgroup     = NULL;
+al_llist_t *stdef        = NULL;
+al_llist_t *thingdef     = NULL;
+al_llist_t *thinggroup   = NULL;
+Wad_name sky_flat;
 
 
 /*
@@ -152,7 +183,22 @@ if (screen_lines == 0)
 // InitSwap must be called before any call to GetMemory(), etc.
 InitSwap ();
 
-// First a quick pass through the command line
+// First detect manually --help and --version
+// because parse_command_line_options() cannot.
+if (argc == 2 && ! strcmp (argv[1], "--help"))
+   {
+   print_usage (stdout);
+   exit (0);
+   }
+if (argc == 2 && ! strcmp (argv[1], "--version"))
+   {
+   puts (what ());
+   puts (config_file_magic);
+   puts (ygd_file_magic);
+   exit (0);
+   }
+
+// Second a quick pass through the command line
 // arguments to detect -?, -f and -help.
 r = parse_command_line_options (argc - 1, argv + 1, 1);
 if (r)
@@ -172,7 +218,7 @@ printf ("%s\n"
 #if defined Y_DOS
 install_dir = spec_path (argv[0]);
 #elif defined Y_UNIX
-install_dir = getenv ("Y_DIR");
+install_dir = getenv ("YADEX_DIR");
 #endif
 
 // The config file provides some values.
@@ -190,22 +236,11 @@ if (! r)
 if (r)
    {
    syntax_error :
-   fprintf (stderr, "Try \"yadex -help\" or \"man yadex\".\n");
+   fprintf (stderr, "Try \"yadex --help\" or \"man yadex\".\n");
    exit (1);
    }
 
-if (Iwad1 == NULL && Iwad2 == NULL)
-   {
-   report_error ("You didn't say where the iwad is.");
-   fprintf (stderr,
-     "To say where doom.wad  is, use \"-i1 <file>\""
-     " or put \"iwad1=<file>\" in yadex.cfg.\n"
-     "To say where doom2.wad is, use \"-i2 <file>\""
-     " or put \"iwad2=<file>\" in yadex.cfg.\n");
-   exit (1);
-   }
-else if (Game != NULL && ! strcmp (Game, "doom")
-      || Game == NULL && Iwad2 == NULL)
+if (Game != NULL && ! strcmp (Game, "doom"))
    {
    if (Iwad1 == NULL)
       {
@@ -216,8 +251,7 @@ else if (Game != NULL && ! strcmp (Game, "doom")
       }
    MainWad = Iwad1;
    }
-else if (Game != NULL && ! strcmp (Game, "doom2")
-      || Game == NULL && Iwad1 == NULL)
+else if (Game != NULL && ! strcmp (Game, "doom2"))
    {
    if (Iwad2 == NULL)
       {
@@ -228,7 +262,7 @@ else if (Game != NULL && ! strcmp (Game, "doom2")
       }
    MainWad = Iwad2;
    }
-else if (! strcmp (Game, "heretic"))
+else if (Game != NULL && ! strcmp (Game, "heretic"))
    {
    if (Iwad3 == NULL)
       {
@@ -239,27 +273,112 @@ else if (! strcmp (Game, "heretic"))
       }
    MainWad = Iwad3;
    }
+else if (Game != NULL && ! strcmp (Game, "hexen"))
+   {
+   if (Iwad4 == NULL)
+      {
+      report_error ("You have to tell me where hexen.wad is.");
+      fprintf (stderr,
+         "Use \"-i4 <file>\" or put \"iwad4=<file>\" in yadex.cfg.\n");
+      exit (1);
+      }
+   MainWad = Iwad4;
+   }
+else if (Game != NULL && ! strcmp (Game, "strife"))
+   {
+   if (Iwad5 == NULL)
+      {
+      report_error ("You have to tell me where strife1.wad is.");
+      fprintf (stderr,
+         "Use \"-i5 <file>\" or put \"iwad5=<file>\" in yadex.cfg.\n");
+      exit (1);
+      }
+   MainWad = Iwad5;
+   }
+else if (Game != NULL && ! strcmp (Game, "doom02"))
+   {
+   if (Iwad6 == NULL)
+      {
+      report_error ("You have to tell me where the Doom alpha 0.2 iwad is.");
+      fprintf (stderr,
+         "Use \"-i6 <file>\" or put \"iwad6=<file>\" in yadex.cfg.\n");
+      exit (1);
+      }
+   MainWad = Iwad6;
+   }
+else if (Game != NULL && ! strcmp (Game, "doom04"))
+   {
+   if (Iwad7 == NULL)
+      {
+      report_error ("You have to tell me where the Doom alpha 0.4 iwad is.");
+      fprintf (stderr,
+         "Use \"-i7 <file>\" or put \"iwad7=<file>\" in yadex.cfg.\n");
+      exit (1);
+      }
+   MainWad = Iwad7;
+   }
+else if (Game != NULL && ! strcmp (Game, "doom05"))
+   {
+   if (Iwad8 == NULL)
+      {
+      report_error ("You have to tell me where the Doom alpha 0.5 iwad is.");
+      fprintf (stderr,
+         "Use \"-i8 <file>\" or put \"iwad8=<file>\" in yadex.cfg.\n");
+      exit (1);
+      }
+   MainWad = Iwad8;
+   }
+else if (Game != NULL && ! strcmp (Game, "doompr"))
+   {
+   if (Iwad9 == NULL)
+      {
+      report_error ("You have to tell me where the Doom press release iwad is.");
+      fprintf (stderr,
+         "Use \"-i9 <file>\" or put \"iwad9=<file>\" in yadex.cfg.\n");
+      exit (1);
+      }
+   MainWad = Iwad9;
+   }
+else if (Game != NULL && ! strcmp (Game, "strife10"))
+   {
+   if (Iwad10 == NULL)
+      {
+      report_error ("You have to tell me where strife1.wad is.");
+      fprintf (stderr,
+         "Use \"-i10 <file>\" or put \"iwad10=<file>\" in yadex.cfg.\n");
+      exit (1);
+      }
+   MainWad = Iwad10;
+   }
 else
    {
-   report_error ("You didn't say for which game you want to edit.");
-   fprintf (stderr, "Use \"-g doom|doom2|heretic\" on the command line\n"
-      "or put \"game=doom|doom2|heretic\" in yadex.cfg.\n");
+   if (Game == NULL)
+      report_error ("You didn't say for which game you want to edit.");
+   else
+      report_error ("Unknown game \"%s\"", Game);
+   fprintf (stderr,
+  "Use \"-g <game>\" on the command line or put \"game=<game>\" in yadex.cfg\n"
+  "where <game> is one of \"doom\", \"doom02\", \"doom04\", \"doom05\","
+  " \"doom2\",\n\"doompr\", \"heretic\", \"hexen\", \"strife\" and "
+  "\"strife10\".\n");
    exit (1);
    }
 if (Debug)
    {
    logfile = fopen (log_file, "a");
    if (logfile == NULL)
-      printf ("Warning: Could not open log file \"%s\"", log_file);
+      warn ("can't open log file \"%s\" (%s)", log_file, strerror (errno));
    LogMessage (": Welcome to Yadex!\n");
    }
 if (Quieter)
-   Quiet = 1;
+   Quiet = true;
 
 // Sanity checks (useful when porting).
 check_types ();
+check_charset ();
 
 // Misc. things done only once.
+cpu_big_endian = native_endianness ();
 add_base_colours ();
 
 // Load game definitions (*.ygd).
@@ -267,7 +386,8 @@ InitGameDefs ();
 LoadGameDefs (Game);
 
 // Load the iwad and the pwads.
-OpenMainWad (MainWad);
+if (OpenMainWad (MainWad))
+   fatal_error ("If you don't give me an iwad, I'll quit. I'm serious.");
 if (PatchWads)
    {
    const char * const *pwad_name;
@@ -277,11 +397,33 @@ if (PatchWads)
 /* sanity check */
 CloseUnusedWadFiles ();
 
-if (welcome_message)
-   print_welcome (stdout);
+// BRANCH 1 : benchmarking (-b)
+if (bench != 0)
+   {
+   benchmark (bench);
+   return 0;  // Exit successfully
+   }
 
-/* all systems go! */
-MainLoop ();
+// BRANCH 2 : normal use ("yadex:" prompt)
+else
+   {
+   if (welcome_message)
+      print_welcome (stdout);
+
+   if (! strcmp (Game, "hexen"))
+      printf (
+   "WARNING: Hexen mode is experimental. Don't expect to be able to do any\n"
+   "real Hexen editing with it. You can edit levels but you can't save them.\n"
+   "And there might be other bugs... BE CAREFUL !\n\n");
+
+   if (! strcmp (Game, "strife"))
+      printf (
+   "WARNING: Strife mode is experimental. Many thing types, linedef types,\n"
+   "etc. are missing or wrong. And be careful, there might be bugs.\n\n");
+
+   /* all systems go! */
+   MainLoop ();
+   }
 
 /* that's all, folks! */
 CloseWadFiles ();
@@ -290,9 +432,10 @@ LogMessage (": The end!\n\n\n");
 if (logfile != NULL)
    fclose (logfile);
 if (remind_to_build_nodes)
-   printf (
-      "-> You have made changes to one or more wads. Don't forget to pass\n"
-      "   them through a nodes builder such as BSP before running them.\n");
+   printf ("\n"
+      "** You have made changes to one or more wads. Don't forget to pass\n"
+      "** them through a nodes builder (E.G. BSP) before running them.\n"
+      "** Like this: \"ybsp foo.wad -o tmp.wad; doom -file tmp.wad\"\n\n");
 return 0;
 }
 
@@ -306,7 +449,7 @@ static int parse_environment_vars ()
 {
 char *value;
 
-value = getenv ("Y_GAME");
+value = getenv ("YADEX_GAME");
 if (value != NULL)
    Game = value;
 return 0;
@@ -327,7 +470,18 @@ if (! Quieter)
    nosound ();
    }
 #elif defined Y_UNIX
-return;
+if (! Quieter)
+#if 0
+   if (dpy)  // FIXME not defined here !
+#else
+   if (1)
+#endif
+      x_bell ();
+   else
+   {
+      putchar ('\a');
+      fflush (stdout);
+   }
 #endif
 }
 
@@ -360,10 +514,29 @@ return;
  */
 void fatal_error (const char *fmt, ...)
 {
-va_list args;
+// With BGI, we have to switch back to text mode
+// before printing the error message so do it now...
+#ifdef Y_BGI
+if (GfxMode)
+   {
+   sleep (1);
+   TermGfx ();
+   }
+#endif
 
+va_list args;
 va_start (args, fmt);
 print_error_message (fmt, args);
+
+// ... on the other hand, with X, we don't have to
+// call TermGfx() before printing so we do it last so
+// that a segfault occuring in TermGfx() does not
+// prevent us from seeing the stderr message.
+#ifdef Y_X11
+if (GfxMode)
+   TermGfx ();  // Don't need to sleep (1) either.
+#endif
+
 // Clean up things and free swap space
 ForgetLevelData ();
 ForgetWTextureNames ();
@@ -392,37 +565,67 @@ print_error_message (fmt, args);
  */
 static void print_error_message (const char *fmt, va_list args)
 {
-//Beep (); Beep ();
-
-// With BGI, we have to switch back to text mode
-// before printing the error message so do it now...
-#ifdef Y_BGI
-if (GfxMode)
-   {
-   sleep (1);
-   TermGfx ();
-   }
-#endif
-
 fflush (stdout);
-fprintf (stderr, "Yadex: Error: ");
+fputs ("Yadex: Error: ", stderr);
 vfprintf (stderr, fmt, args);
 fputc ('\n', stderr);
+fflush (stderr);
 if (Debug && logfile != NULL)
    {
-   fprintf (logfile, "Yadex: Error: ");
+   fputs ("Yadex: Error: ", logfile);
    vfprintf (logfile, fmt, args);
    fputc ('\n', logfile);
+   fflush (logfile);
    }
+}
 
-// ... on the other hand, with X, we don't have to
-// call TermGfx() before printing so we do it last so
-// that a segfault occuring in TermGfx() does not
-// prevent us from seeing the stderr message.
-#ifdef Y_X11
-if (GfxMode)
-   TermGfx ();  // Don't need to sleep (1) either.
-#endif
+
+/*
+ *	nf_bug
+ *	Report about a non-fatal bug to stderr. The message
+ *	should not expand to more than 80 characters.
+ */
+void nf_bug (const char *fmt, ...)
+{
+static bool first_time = 1;
+static int repeats = 0;
+static char msg_prev[81];
+char msg[81];
+
+va_list args;
+va_start (args, fmt);
+y_vsnprintf (msg, sizeof msg, fmt, args);
+if (first_time || strncmp (msg, msg_prev, sizeof msg))
+   {
+   fflush (stdout);
+   if (repeats)
+      {
+      fprintf (stderr, "Yadex: Bug: Previous message repeated %d times\n",
+	    repeats);
+      repeats = 0;
+      }
+
+   fprintf (stderr, "Yadex: Bug: %s\n", msg);
+   fflush (stderr);
+   if (first_time)
+      {
+      fputs ("REPORT ALL \"Bug:\" MESSAGES TO THE MAINTAINER !\n", stderr);
+      first_time = 0;
+      }
+   strncpy (msg_prev, msg, sizeof msg_prev);
+   }
+else
+   {
+   repeats++;  // Same message as above
+   if (repeats == 10)
+      {
+      fflush (stdout);
+      fprintf (stderr, "Yadex: Bug: Previous message repeated %d times\n",
+	    repeats);
+      fflush (stderr);
+      repeats = 0;
+      }
+   }
 }
 
 
@@ -448,7 +651,6 @@ if (Debug && logfile != NULL)
       fprintf (logfile, "%s", tstr);
       }
    vfprintf (logfile, logstr, args);
-   va_end (args);
    fflush (logfile);  /* AYM 19971031 */
    }
 }
@@ -469,71 +671,86 @@ WadPtr wad;
 for (;;)
    {
    /* get the input */
-   printf ("\n[? for help]> ");
-   fgets (input, sizeof input, stdin);
+   printf ("yadex: ");
+   if (! fgets (input, sizeof input, stdin))
+      {
+      puts ("q");
+      break;
+      }
+
    /* Strip the trailing '\n' */
    if (strlen (input) > 0 && input[strlen (input) - 1] == '\n')
       input[strlen (input) - 1] = '\0';
-   printf ("\n");
-   /* strupr (input);  AYM 1998-06-13 */
 
    /* eat the white space and get the first command word */
    com = strtok (input, " ");
 
    /* user just hit return */
    if (com == NULL)
-      printf ("[Please enter a command or ? for help.]\n");
+      printf ("Please enter a command or ? for help.\n");
 
    /* user inputting for help */
-   else if (!strcmp (com, "?"))
+   else if (!strcmp (com, "?") || !strcmp (com, "h") || !strcmp (com, "help"))
       {
-      printf ("?                                 --"
-              " to display this text\n");
+      printf ("? | h | help                      --"
+              " display this text\n");
       printf ("b[uild] <WadFile>                 --"
-              " to build a new iwad\n");
+              " build a new iwad\n");
       printf ("c[reate] <levelname>              --"
-              " to create and edit a new (empty) level\n");
+              " create and edit a new (empty) level\n");
       printf ("d[ump] <DirEntry> [outfile]       --"
-              " to dump a directory entry in hex\n");
+              " dump a directory entry in hex\n");
       printf ("e[dit] <levelname>                --"
-              " to edit a game level saving results to\n");
+              " edit a game level saving results to\n");
       printf ("                                          a patch wad file\n");
       printf ("g[roup] <WadFile>                 --"
-              " to group all patch wads in a file\n");
+              " group all patch wads in a file\n");
       printf ("i[nsert] <RawFile> <DirEntry>     --"
-              " to insert a raw file in a patch wad file\n");
+              " insert a raw file in a patch wad file\n");
       printf ("l[ist] <WadFile> [outfile]        --"
-              " to list the directory of a wadfile\n");
+              " list the directory of a wadfile\n");
       printf ("m[aster] [outfile]                --"
-              " to list the master directory\n");
+              " list the master directory\n");
       printf ("make_gimp_palette <outfile>       --"
-              " to generate a gimp palette file from\n"
+              " generate a gimp palette file from\n"
+              "                                    "
+              " entry 0 of lump PLAYPAL.\n");
+      printf ("make_palette_ppm <outfile>        --"
+              " generate a palette image from\n"
               "                                    "
               " entry 0 of lump PLAYPAL.\n");
       printf ("q[uit]                            --"
-              " to quit\n");
+              " quit\n");
       printf ("r[ead] <WadFile>                  --"
-              " to read a new wad patch file\n");
+              " read a new wad patch file\n");
       printf ("s[ave] <DirEntry> <WadFile>       --"
-              " to save one object to a separate file\n");
+              " save one object to a separate file\n");
       printf ("set                               --"
-              " to list all options and their values\n");
-      printf ("v[iew] [SpriteName]               --"
-              " to display the sprites\n");
+              " list all options and their values\n");
+      printf ("v[iew] [<spritename>]             --"
+              " display the sprites\n");
+      printf ("viewflat [<flatname>]             --"
+	      " flat viewer\n");
+      printf ("viewpal                           --"
+	      " palette viewer\n");
+      printf ("viewpat [<patchname>]             --"
+	      " patch viewer\n");
+      printf ("viewtex [<texname>]               --"
+	      " texture viewer\n");
       printf ("w[ads]                            --"
-              " to display the open wads\n");
+              " display the open wads\n");
       printf ("x[tract] <DirEntry> <RawFile>     --"
-              " to save (extract) one object to a raw file\n");
+              " save (extract) one object to a raw file\n");
       }
 
    /* user asked for list of open wad files */
    else if (!strcmp (com, "wads") || !strcmp (com, "w"))
       {
-      printf ("%-20s  IWAD  (Main wad file)\n", WadFileList->filename);
+      printf ("%-40s  Iwad\n", WadFileList->filename);
       for (wad = WadFileList->next; wad; wad = wad->next)
 	 {
-	 printf ("%-20s  PWAD  (Patch wad file for %.8s)\n",
-	  wad->filename, wad->directory[0].name);
+	 printf ("%-40s  Pwad (%.*s)\n",
+	  wad->filename, (int) WAD_NAME, wad->directory[0].name);
 	 }
       }
 
@@ -542,9 +759,6 @@ for (;;)
       {
       if (! Registered)
 	 printf ("Remember to register your copy of the game !\n");
-#ifdef OLD
-      printf ("Goodbye...\n");
-#endif
       break;
       }
 
@@ -553,37 +767,52 @@ for (;;)
          || !strcmp (com, "create") || !strcmp (com, "c"))
       {
       const int newlevel = ! strcmp (com, "create") || ! strcmp (com, "c");
-
+      char *level_name = 0;
       com = strtok (NULL, " ");
-      if (com == NULL)
-	 {
-	 printf ("Which level ?\n");
-         continue;
-	 }
-      char *level_name = find_level (com);
-      if (level_name == error_invalid)
-         printf ("\"%s\" is not a valid level name.\n", com);
-      else if (level_name == error_none)
-         printf ("Neither E%dM%d nor MAP%02d exist.\n",
-            atoi (com) / 10, atoi (com) % 10, atoi (com));
-      else if (level_name == error_non_unique)
-         printf ("Both E%dM%d and MAP%02d exist. Use an unambiguous name.\n",
-            atoi (com) / 10, atoi (com) % 10, atoi (com));
-      else if (level_name == NULL)
-         {
-         printf ("Level %s not found.", com);
-         if (tolower (*com) == 'e' && ! strcmp (Game, "doom2")
-            || tolower (*com) == 'm' && strcmp (Game, "doom2"))
-            printf (" You are in %s mode.", Game);
-         else if (tolower (*com) == 'e' && com[1] > '1' && ! Registered)
-            printf (" You have the shareware iwad.");
-         putchar ('\n');
-         }
+      if (com == 0)
+	 if (! newlevel)
+	    {
+	    printf ("Which level ?\n");
+	    continue;
+	    }
+         else
+	    level_name = 0;
       else
-         {
-         EditLevel (level_name, newlevel);
+	 {
+	 level_name = find_level (com);
+	 if (level_name == error_invalid)
+	    {
+	    printf ("\"%s\" is not a valid level name.\n", com);
+	    continue;
+	    }
+	 else if (level_name == error_none)
+	    {
+	    printf ("Neither E%dM%d nor MAP%02d exist.\n",
+	       atoi (com) / 10, atoi (com) % 10, atoi (com));
+	    continue;
+	    }
+	 else if (level_name == error_non_unique)
+	    {
+	    printf ("Both E%dM%d and MAP%02d exist. Use an unambiguous name.\n",
+	       atoi (com) / 10, atoi (com) % 10, atoi (com));
+	    continue;
+	    }
+	 else if (level_name == NULL)
+	    {
+	    printf ("Level %s not found.", com);
+	    // Hint absent-minded users
+	    if (tolower (*com) == 'e' && yg_level_name == YGLN_MAP01
+	       || tolower (*com) == 'm' && yg_level_name == YGLN_E1M1)
+	       printf (" You are in %s mode.", Game);
+	    else if (tolower (*com) == 'e' && com[1] > '1' && ! Registered)
+	       printf (" You have the shareware iwad.");
+	    putchar ('\n');
+	    continue;
+	    }
+	 }
+      EditLevel (level_name, newlevel);
+      if (level_name)
          free (level_name);
-         }
       }
 
    /* user asked to build a new main wad file */
@@ -592,7 +821,7 @@ for (;;)
       com = strtok (NULL, " ");
       if (com == NULL)
 	 {
-	 printf ("[Wad file name argument missing.]\n");
+	 printf ("Wad file name argument missing.\n");
 	 continue;
 	 }
       for (wad = WadFileList; wad; wad = wad->next)
@@ -600,7 +829,7 @@ for (;;)
 	    break;
       if (wad)
 	 {
-	 printf ("[File \"%s\" is opened and cannot be overwritten.]\n", com);
+	 printf ("File \"%s\" is opened and cannot be overwritten.\n", com);
 	 continue;
 	 }
       BuildNewMainWad (com, 0);
@@ -611,14 +840,14 @@ for (;;)
       {
       if (WadFileList->next == NULL || WadFileList->next->next == NULL)
 	 {
-	 printf ("[You need at least two open wad files "
-	   "if you want to group them.]\n");
+	 printf ("You need at least two open wad files "
+	   "if you want to group them.\n");
 	 continue;
 	 }
       com = strtok (NULL, " ");
       if (com == NULL)
 	 {
-	 printf ("[Wad file name argument missing.]\n");
+	 printf ("Wad file name argument missing.\n");
 	 continue;
 	 }
       for (wad = WadFileList; wad; wad = wad->next)
@@ -626,7 +855,7 @@ for (;;)
 	    break;
       if (wad)
 	 {
-	 printf ("[File \"%s\" is opened and cannot be overwritten.]\n", com);
+	 printf ("File \"%s\" is opened and cannot be overwritten.\n", com);
 	 continue;
 	 }
       BuildNewMainWad (com, 1);
@@ -638,7 +867,7 @@ for (;;)
       com = strtok (NULL, " ");
       if (com == NULL)
 	 {
-	 printf ("[Wad file name argument missing.]\n");
+	 printf ("Wad file name argument missing.\n");
 	 continue;
 	 }
       for (wad = WadFileList; wad; wad = wad->next)
@@ -646,7 +875,7 @@ for (;;)
 	    break;
       if (wad == NULL)
 	 {
-	 printf ("[Wad file \"%s\" is not open.]\n", com);
+	 printf ("Wad file \"%s\" is not open.\n", com);
 	 continue;
 	 }
       out = strtok (NULL, " ");
@@ -695,6 +924,30 @@ for (;;)
       make_gimp_palette (0, out);
       }
 
+   // make_palette_ppm
+   else if (! strcmp (com, "make_palette_ppm"))
+      {
+      out = strtok (NULL, "");
+      if (out == NULL)
+	 {
+	 printf ("Output file name argument missing.\n");
+	 continue;
+	 }
+      make_palette_ppm (0, out);
+      }
+
+   // make_palette_ppm
+   else if (! strcmp (com, "mp2"))
+      {
+      out = strtok (NULL, "");
+      if (out == NULL)
+	 {
+	 printf ("Output file name argument missing.\n");
+	 continue;
+	 }
+      make_palette_ppm_2 (0, out);
+      }
+
    /* user asked to list all options and their values */
    else if (! strcmp (com, "set"))
       {
@@ -725,7 +978,7 @@ for (;;)
       com = strtok (NULL, " ");
       if (com == NULL)
 	 {
-	 printf ("[Object name argument missing.]\n");
+	 printf ("Object name argument missing.\n");
 	 continue;
 	 }
       out = strtok (NULL, " ");
@@ -743,13 +996,135 @@ for (;;)
 	 DumpDirectoryEntry (stdout, com);
       }
 
-   /* user asked to view the sprites */
+   // "v"/"view" - view the sprites
    else if (!strcmp (com, "view") || !strcmp (com, "v"))
       {
-      InitGfx ();
-      com = strtok (NULL, " ");
-      ChooseSprite (-1, -1, "Sprite viewer", com);
+      if (InitGfx ())
+	 goto v_end;
+      init_input_status ();
+      do
+	 get_input_status ();
+      while (is.key != YE_EXPOSE);
+      force_window_not_pixmap ();  // FIXME quick hack
+      {
+      Lump_list list;
+      wad_res.sprites.list (list);
+      char buf[WAD_PIC_NAME + 1];
+      const char *sprite = strtok (NULL, " ");
+      *buf = '\0';
+      if (sprite != 0)
+      {
+	strncat (buf, sprite, sizeof buf - 1);
+	for (char *p = buf; *p != '\0'; p++)
+	  *p = toupper (*p);
+      }
+      InputNameFromListWithFunc (-1, -1, "Sprite viewer", list.size (),
+	list.data (), 10, buf, 320, 200, display_pic,
+	HOOK_DISP_SIZE | HOOK_SPRITE);
+      }
       TermGfx ();
+      v_end:;
+      }
+
+   // "viewflat" - view the flats
+   else if (! strcmp (com, "viewflat"))
+      {
+      if (InitGfx ())
+	goto viewflat_end;
+      init_input_status ();
+      do
+	 get_input_status ();
+      while (is.key != YE_EXPOSE);
+      com = strtok (NULL, " ");
+      force_window_not_pixmap ();  // FIXME quick hack
+      char buf[WAD_FLAT_NAME + 1];
+      *buf = '\0';
+      if (com != 0)
+	strncat (buf, com, sizeof buf - 1);
+      ReadFTextureNames ();
+      {
+      char **flat_names =
+	(char **) GetMemory (NumFTexture * sizeof *flat_names);
+      for (size_t n = 0; n < NumFTexture; n++)
+	flat_names[n] = flat_list[n].name;
+      ChooseFloorTexture (-1, -1, "Flat viewer",
+	NumFTexture, flat_names, buf);
+      FreeMemory (flat_names);
+      }
+      ForgetFTextureNames ();
+      TermGfx ();
+      viewflat_end:;
+      }
+
+   // "viewpal" - view the palette (PLAYPAL and COLORMAP)
+   else if (! strcmp (com, "viewpal"))
+      {
+      if (InitGfx ())
+	goto viewpal_end;
+      init_input_status ();
+      do
+	 get_input_status ();
+      while (is.key != YE_EXPOSE);
+      force_window_not_pixmap ();  // FIXME quick hack
+      {
+      Palette_viewer pv;
+      pv.run ();
+      }
+      TermGfx ();
+      viewpal_end:;
+      }
+   
+   // "viewpat" - view the patches
+   else if (! strcmp (com, "viewpat"))
+      {
+      if (InitGfx ())
+	goto viewpat_end;
+      init_input_status ();
+      do
+	 get_input_status ();
+      while (is.key != YE_EXPOSE);
+      com = strtok (NULL, " ");
+      force_window_not_pixmap ();  // FIXME quick hack
+      patch_dir.refresh (MasterDir);
+      {
+	 char buf[WAD_NAME + 1];
+	 *buf = '\0';
+	 if (com != 0)
+	   strncat (buf, com, sizeof buf - 1);
+	 Patch_list pl;
+	 patch_dir.list (pl);
+	 InputNameFromListWithFunc (-1, -1, "Patch viewer", pl.size (),
+	       pl.data (), 10, buf, 256, 256, display_pic,
+	       HOOK_DISP_SIZE | HOOK_PATCH);
+      }
+      TermGfx ();
+      viewpat_end:;
+      }
+
+   // "viewtex" - view the textures
+   else if (! strcmp (com, "viewtex"))
+      {
+      if (InitGfx ())
+	goto viewtex_end;
+      init_input_status ();
+      do
+	 get_input_status ();
+      while (is.key != YE_EXPOSE);
+      com = strtok (NULL, " ");
+      force_window_not_pixmap ();  // FIXME quick hack
+      patch_dir.refresh (MasterDir);
+      {
+	 char buf[WAD_TEX_NAME + 1];
+	 *buf = '\0';
+	 if (com != 0)
+	   strncat (buf, com, sizeof buf - 1);
+	 ReadWTextureNames ();
+	 ChooseWallTexture (-1, -1, "Texture viewer", NumWTexture, WTexture,
+	   buf);
+	 ForgetWTextureNames ();
+      }
+      TermGfx ();
+      viewtex_end:;
       }
 
    /* user asked to save an object to a separate pwad file */
@@ -758,18 +1133,18 @@ for (;;)
       com = strtok (NULL, " ");
       if (com == NULL)
 	 {
-	 printf ("[Object name argument missing.]\n");
+	 printf ("Object name argument missing.\n");
 	 continue;
 	 }
-      if (strlen (com) > 8 || strchr (com, '.') != NULL)
+      if (strlen (com) > WAD_NAME || strchr (com, '.') != NULL)
 	 {
-	 printf ("[Invalid object name.]\n");
+	 printf ("Invalid object name.\n");
 	 continue;
 	 }
       out = strtok (NULL, " ");
       if (out == NULL)
 	 {
-	 printf ("[Wad file name argument missing.]\n");
+	 printf ("Wad file name argument missing.\n");
 	 continue;
 	 }
       for (wad = WadFileList; wad; wad = wad->next)
@@ -777,8 +1152,8 @@ for (;;)
 	    break;
       if (wad)
 	 {
-	 printf ("[This wad file is already in use. "
-	    "You may not overwrite it.]\n");
+	 printf ("This wad file is already in use. "
+	    "You may not overwrite it.\n");
 	 continue;
 	 }
       printf ("Saving directory entry data to \"%s\".\n", out);
@@ -794,18 +1169,18 @@ for (;;)
       com = strtok (NULL, " ");
       if (com == NULL)
 	 {
-	 printf ("[Raw file name argument missing.]\n");
+	 printf ("Raw file name argument missing.\n");
 	 continue;
 	 }
       out = strtok (NULL, " ");
       if (out == NULL)
 	 {
-	 printf ("[Object name argument missing.]\n");
+	 printf ("Object name argument missing.\n");
 	 continue;
 	 }
-      if (strlen (out) > 8 || strchr (out, '.') != NULL)
+      if (strlen (out) > WAD_NAME || strchr (out, '.') != NULL)
 	 {
-	 printf ("[Invalid object name.]\n");
+	 printf ("Invalid object name.\n");
 	 continue;
 	 }
       if ((raw = fopen (com, "rb")) == NULL)
@@ -818,8 +1193,8 @@ for (;;)
 	    break;
       if (wad)
 	 {
-	 printf ("[This wad file is already in use (%s). "
-	    "You may not overwrite it.]\n", input);
+	 printf ("This wad file is already in use (%s). "
+	    "You may not overwrite it.\n", input);
 	 continue;
 	 }
       printf ("Including new object %s in \"%s\".\n", out, input);
@@ -838,18 +1213,18 @@ for (;;)
       com = strtok (NULL, " ");
       if (com == NULL)
 	 {
-	 printf ("[Object name argument missing.]\n");
+	 printf ("Object name argument missing.\n");
 	 continue;
 	 }
-      if (strlen (com) > 8 || strchr (com, '.') != NULL)
+      if (strlen (com) > WAD_NAME || strchr (com, '.') != NULL)
 	 {
-	 printf ("[Invalid object name.]\n");
+	 printf ("Invalid object name.\n");
 	 continue;
 	 }
       out = strtok (NULL, " ");
       if (out == NULL)
 	 {
-	 printf ("[Raw file name argument missing.]\n");
+	 printf ("Raw file name argument missing.\n");
 	 continue;
 	 }
       for (wad = WadFileList; wad; wad = wad->next)
@@ -857,7 +1232,7 @@ for (;;)
 	    break;
       if (wad)
 	 {
-	 printf ("[You may not overwrite an opened wad file with raw data.]\n");
+	 printf ("You may not overwrite an opened wad file with raw data.\n");
 	 continue;
 	 }
       printf ("Saving directory entry data to \"%s\".\n", out);
@@ -869,7 +1244,7 @@ for (;;)
 
    /* unknown command */
    else
-      printf ("[Unknown command \"%s\"!]\n", com);
+      printf ("Unknown command \"%s\"!\n", com);
    }
 }
 
@@ -902,7 +1277,7 @@ for (size_t n = 0; n < NCOLOURS; n++)
  //     c.set (0x50, 0x46, 0x40);  //0xc0 //0xa8 //0x80
       c.set (0x48, 0x42, 0x3c);  //0xc0 //0xa8 //0x80
    else if (n == WINBG_DARK)
-      c.set (0x20, 0x1b, 0x16);  //0x40 //0x38 //0x30
+      c.set (0x20, 0x1b, 0x12);  //0x40 //0x38 //0x30
    else if (n == WINFG)
       c.set (0xa0, 0xa0, 0xa0);
    else if (n == WINFG_DIM)
@@ -910,17 +1285,37 @@ for (size_t n = 0; n < NCOLOURS; n++)
    else if (n == WINBG_HL)
       c.set (0x58, 0x50, 0x48);
    else if (n == WINFG_HL)
-      c.set (0xc0, 0xc0, 0xc0);
+      c.set (0xd0, 0xd0, 0xd0);
    else if (n == WINFG_DIM_HL)
       c.set (0x70, 0x70, 0x70);
    else if (n == GRID1)
-      c.set (0, 0, 0x60);
-   else if (n == GRID2)
-      c.set (0, 0, 0xa0);
-   else if (n == GRID3)
-      c.set (0, 0, 0xe0);
+      c.set (0, 0, 0xc0);
+   else if (n == GRID2H)
+      c.set (0, 0, 0x30);
+   else if (n == GRID2V)
+      c.set (0, 0, 0x40);
+   else if (n == GRID3H)
+      c.set (0, 0, 0x50);
+   else if (n == GRID3V)
+      c.set (0, 0, 0x70);
+   else if (n == GRID4H)
+      c.set (0, 0, 0x80);
+   else if (n == GRID4V)
+      c.set (0, 0, 0xc0);
    else if (n == WINFGLABEL)  // Text in window, a bit dimmer
       c.set (0x88, 0x88, 0x88);
+   else if (n == LINEDEF_NO)
+      c.set (0x40, 0xd0, 0xf0);
+   else if (n == SECTOR_NO)
+      c.set (0x40, 0xd0, 0xf0);
+   else if (n == THING_NO)
+      c.set (0x40, 0xd0, 0xf0);
+   else if (n == VERTEX_NO)
+      c.set (0x40, 0xd0, 0xf0);
+   else if (n == CLR_ERROR)
+      c.set (0xff, 0, 0);
+   else if (n == THING_REM)
+      c.set (0x40, 0x40, 0x40);
    else
       fatal_error ("Wrong acn %d", n);
 
